Package: pagedown
Type: Package
Title: Paginate the HTML Output of R Markdown with CSS for Print
Version: 0.25
Authors@R: c(
  person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666")),
  person("Romain", "Lesur", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-0721-5595")),
  person("Christophe", "Dervieux", role = c("ctb"), comment = c(ORCID = "0000-0003-4474-2498")),
  person("Brent", "Thorne", role = "aut", comment = c(ORCID = "0000-0002-1099-3857")),
  person("Xianying", "Tan", role = "aut", comment = c(ORCID = "0000-0002-6072-3521")),
  person("Atsushi", "Yasumoto", role = c("ctb"), comment = c(ORCID = "0000-0002-8335-495X")),
  person(given = "Posit Software, PBC", role = c("cph", "fnd")),
  person("Adam", "Hyde", role = "ctb", comment = "paged.js in resources/js/"),
  person("Min-Zhong", "Lu", role = "ctb", comment = "resume.css in resources/css/"),
  person("Zulko", role = "ctb", comment = "poster-relaxed.css in resources/css/"),
  person()
  )
Description: Use the paged media properties in CSS and the JavaScript
  library 'paged.js' to split the content of an HTML document into discrete
  pages. Each page can have its page size, page numbers, margin boxes, and
  running headers, etc. Applications of this package include books, letters,
  reports, papers, business cards, resumes, and posters.
Depends: R (>= 3.5.0)
Imports: rmarkdown (>= 2.13), bookdown (>= 0.8), htmltools, jsonlite,
        later (>= 1.0.0), processx, servr (>= 0.31), httpuv, xfun,
        websocket
Suggests: promises, testit, xaringan, pdftools, revealjs, covr, xml2
License: MIT + file LICENSE
URL: https://github.com/rstudio/pagedown
BugReports: https://github.com/rstudio/pagedown/issues
SystemRequirements: Pandoc (>= 2.2.3)
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-27 15:43:53 UTC; yihui
Author: Yihui Xie [aut, cre] (ORCID: <https://orcid.org/0000-0003-0645-5666>),
  Romain Lesur [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-0721-5595>),
  Christophe Dervieux [ctb] (ORCID:
    <https://orcid.org/0000-0003-4474-2498>),
  Brent Thorne [aut] (ORCID: <https://orcid.org/0000-0002-1099-3857>),
  Xianying Tan [aut] (ORCID: <https://orcid.org/0000-0002-6072-3521>),
  Atsushi Yasumoto [ctb] (ORCID: <https://orcid.org/0000-0002-8335-495X>),
  Posit Software, PBC [cph, fnd],
  Adam Hyde [ctb] (paged.js in resources/js/),
  Min-Zhong Lu [ctb] (resume.css in resources/css/),
  Zulko [ctb] (poster-relaxed.css in resources/css/)
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2026-08-28 07:20:09 UTC

# Generated by roxygen2: do not edit by hand

export(book_crc)
export(business_card)
export(chrome_print)
export(find_chrome)
export(html_letter)
export(html_paged)
export(html_resume)
export(jss_paged)
export(poster_jacobs)
export(poster_relaxed)
export(thesis_paged)
import(stats)
import(utils)

# pagedown <img src="man/figures/logo.png" align="right" height="138" />

<!-- badges: start -->
[![R-CMD-check](https://github.com/rstudio/pagedown/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rstudio/pagedown/actions/workflows/R-CMD-check.yaml)
[![Downloads from the RStudio CRAN mirror](https://cranlogs.r-pkg.org/badges/pagedown)](https://cran.r-project.org/package=pagedown)
<!-- badges: end -->

Paginate the HTML Output of R Markdown with CSS for Print. You only need a modern web browser (e.g., Google Chrome or Microsoft Edge) to generate PDF. No need to install LaTeX to get beautiful PDFs.

This R package stands on the shoulders of two giants to support typesetting with CSS for R Markdown documents: [Paged.js](https://pagedjs.org) and [ReLaXed](https://github.com/RelaxedJS/ReLaXed) (we only borrowed some CSS from the ReLaXed repo and didn't really use the Node package).

You may install this package from Github:

```r
# install.packages("pak")
pak::pak('rstudio/pagedown')
```

This package requires a recent version of Pandoc (>= 2.2.3). If you use RStudio, you are recommended to install the [latest version](https://docs.posit.co/ide/user/#rstudio-ide-oss-downloads) (>= 1.2.1335), which has bundled Pandoc 2.x, otherwise you need to install Pandoc separately.

Below are some existing R Markdown output formats and examples.

## Paged HTML documents (`pagedown::html_paged`)

[![A paged HTML document](https://user-images.githubusercontent.com/163582/47673682-58b11880-db83-11e8-87fd-b5e753af7288.png)](https://pagedown.rbind.io)

### Thesis (`pagedown::thesis_paged`)

[![A paged HTML thesis](https://user-images.githubusercontent.com/19177171/60126180-b129cb80-978d-11e9-8e32-7ba48f303b6f.png)](https://pagedown.rbind.io/thesis-paged/)

### Journal of Statistical Software article (`pagedown::jss_paged`)

[![A JSS article](https://user-images.githubusercontent.com/19177171/51005498-5b46cb80-153f-11e9-9026-4b50a9f3d3f1.png)](https://pagedown.rbind.io/jss-paged/)

## Resume (`pagedown::html_resume`)

[![An HTML resume](https://user-images.githubusercontent.com/163582/46879762-7a34a500-ce0c-11e8-87e3-496f3577ff05.png)](https://pagedown.rbind.io/html-resume/)

You may want to check out [the **datadrivencv** package](https://github.com/nstrayer/datadrivencv), which allows you to build a CV using data from a spreadsheet and `pagedown::html_resume`.

## Posters

### `pagedown::poster_relaxed`

[![A poster of the ReLaXed style](https://user-images.githubusercontent.com/163582/47672385-e12dba00-db7f-11e8-92de-af94d5bab12f.jpg)](https://pagedown.rbind.io/poster-relaxed/)

### `pagedown::poster_jacobs`

[![A poster of the Jacobs University style](https://user-images.githubusercontent.com/163582/49780277-7b326780-fcd3-11e8-9eb6-69e46292158c.png)](https://pagedown.rbind.io/poster-jacobs/)

Want more poster styles? You may take a look at [Brent Thorne's **posterdown** package](https://github.com/brentthorne/posterdown).

## Business cards (`pagedown::business_card`)

[![A business card](https://user-images.githubusercontent.com/163582/47741877-68933000-dc49-11e8-94f8-92724b67e9a6.png)](https://pagedown.rbind.io/business-card/)

## Letters (`pagedown::html_letter`)

[![A letter in HTML](https://user-images.githubusercontent.com/163582/47872372-61e8f200-dddc-11e8-839b-d8e8ef8f51eb.png)](https://pagedown.rbind.io/html-letter)

## Other examples

- Nick Strayer's CV: https://github.com/nstrayer/cv (also includes instructions on how to build your own CV).

- "Template of Exec Summaries with pagedown" by Joshua David Barillas: https://github.com/jdbarillas/executive_summary

- Ulrik Lyngs's CVs: https://ulyngs.github.io/pagedown-cv/

- Jiena Gu McLellan's CV: https://github.com/jienagu/Jiena_McLellan_CV

- Thomas Vroylandt's template for the French Scouts and Guides Association: https://github.com/tvroylandt/sgdf_pagedown

- Beatriz Milz's CV: https://beatrizmilz.github.io/resume/resume.html

- **pagedreport** by Thomas Vroylandt and David Keyes which helps using **pagedown** with some templates proposals: https://github.com/rfortherestofus/pagedreport

- Jodavid Ferreira's CV (in Portuguese) : https://jodavid.github.io/cv/ (source: https://github.com/jodavid/cv)

## Authors and contributors

The main authors of this package are Yihui Xie (RStudio) and Romain Lesur. Romain has received a grant from the Shuttleworth Foundation for his work on both Paged.js and **pagedown**.

[![Shuttleworth Funded](https://user-images.githubusercontent.com/163582/49319242-72ff4e80-f4c1-11e8-89fe-d8749355d261.jpg)](https://www.shuttleworthfoundation.org)

You can find [the full list of contributors of **pagedown** here](https://github.com/rstudio/pagedown/graphs/contributors). We always welcome new contributions. In particular, if you are familiar with CSS, we'd love to include your contributions of more creative and beautiful CSS stylesheets in this package. It is also very helpful if you don't know CSS but just tell us the creative and beautiful web pages you have seen, since other CSS experts may be able to port them into **pagedown**.

library(testit)
test_format <- function(template, output_options = NULL, skip = NULL) {

  # don't run on CRAN due to complicated dependencies (Pandoc)
  if (!identical(Sys.getenv("NOT_CRAN"), "true")) return()
  # skip if requested
  if (!is.null(skip) && isTRUE(skip)) return()

  # work in a temp directory
  dir <- tempfile()
  dir.create(dir)
  oldwd <- setwd(dir)
  on.exit({
    setwd(oldwd)
    unlink(dir, recursive = TRUE)
  }, add = TRUE)

  # create a draft of the format
  testdoc <- paste0(template,".Rmd")
  rmarkdown::draft(
    testdoc, template,
    package = "pagedown",
    create_dir = FALSE, edit = FALSE
  )

  message('Rendering the template for ', template, ' format...',
          if(!is.null(output_options)) " (with output options)")
  output_file <- rmarkdown::render(testdoc, output_options = output_options, quiet = TRUE)
  assert(paste(template, "format works"), {
    file.exists(output_file)
  })
}

templates <- dir(system.file("rmarkdown", "templates", package = "pagedown"))

for (template in templates) test_format(template)

#' Create a resume in HTML
#'
#' This output format is based on Min-Zhong Lu's HTML/CSS in the Github repo
#' \url{https://github.com/mnjul/html-resume}. See
#' \url{https://pagedown.rbind.io/html-resume/} for an example.
#' @param ...,css,template,number_sections,fig_caption See
#'   \code{\link{html_paged}()}.
#' @return An R Markdown output format.
#' @export
html_resume = function(
  ..., css = 'resume', template = pkg_resource('html', 'resume.html'),
  number_sections = FALSE, fig_caption = FALSE
) {
  html_format(
    ..., css = css, template = template, theme = NULL, number_sections = number_sections,
    fig_caption = fig_caption, .pagedjs = TRUE,
    .dependencies = list(rmarkdown::html_dependency_font_awesome())
  )
}

# TODO: port this resume style https://github.com/ohrie/relaxed-resume

library(testit)

assert('Page breaks work', {
  f = print_pdf('test-page-breaks.Rmd')
  (identical(pdftools::pdf_info(f)$pages, 5L))
})

assert('lot / lof are inserted correctly', {
  # This test is tied to skeleton.Rmd for html-paged template
  # Check when skeleton.Rmd is modified
  if (xfun::loadable("xml2")) {
    dir.create(tmp_dir <- tempfile())
    xfun::in_dir(tmp_dir, {
      # use included template
      rmarkdown::draft("test.Rmd", "html-paged", "pagedown", edit = FALSE)
      res = rmarkdown::render("test.Rmd", quiet = TRUE)
    })
    content <- xml2::read_html(res, encoding = "UTF-8")
    # correctly in TOC
    toc <- xml2::xml_find_all(content, "//div[@id='TOC']")
    toc_lot <- xml2::xml_find_all(toc, "ul/li/a[@href='#LOT']")
    (length(toc_lot) == 1)
    (xml2::xml_text(toc_lot) == "List of Tables")
    toc_lof <- xml2::xml_find_all(toc, "ul/li/a[@href='#LOF']")
    (length(toc_lof) == 1)
    (xml2::xml_text(toc_lof) == "List of Figures")
    # Correctly in top body
    main <- xml2::xml_find_all(content, "//div[contains(@class, 'main')]")
    divs <- xml2::xml_find_all(main, "div[@id]")
    (xml2::xml_attr(divs[1], 'id') == "LOT")
    (length(xml2::xml_find_all(divs[1], 'ul/li/a[contains(@href, "md-tab")]')) == 1L)
    (length(xml2::xml_find_all(divs[1], 'ul/li/a[contains(@href, "simple-tab")]')) == 1L)
    (xml2::xml_attr(divs[2], 'id') == "LOF")
    (length(xml2::xml_find_all(divs[2], 'ul/li/a[contains(@href, "md-fig")]')) == 1L)
    (length(xml2::xml_find_all(divs[2], 'ul/li/a[contains(@href, "simple-graphic")]')) == 1L)

    # Change titles
    xfun::in_dir(tmp_dir, {
      xfun::gsub_file("test.Rmd", pattern = "^---$",
                      replacement = "---\nlot-title: Custom LOT\nlof-title: Custom LOF")
      res = rmarkdown::render("test.Rmd", quiet = TRUE)
    })
    content <- xml2::read_html(res)
    # correctly in TOC
    toc <- xml2::xml_find_all(content, "//div[@id='TOC']")
    toc_lot <- xml2::xml_find_all(toc, "ul/li/a[@href='#LOT']")
    (length(toc_lot) == 1)
    (xml2::xml_text(toc_lot) == "Custom LOT")
    toc_lof <- xml2::xml_find_all(toc, "ul/li/a[@href='#LOF']")
    (length(toc_lof) == 1)
    (xml2::xml_text(toc_lof) == "Custom LOF")
    # Correctly in top body
    main <- xml2::xml_find_all(content, "//div[contains(@class, 'main')]")
    h1 <- xml2::xml_find_all(main, "div/h1")
    (xml2::xml_text(h1[1]) == "Custom LOT")
    (xml2::xml_text(h1[2]) == "Custom LOF")

    # cleaning
    unlink(tmp_dir, recursive = TRUE)
  }
})

pkg_resource = function(...) {
  system.file('resources', ..., package = 'pagedown', mustWork = TRUE)
}

lua_filters = function(...) {
  c(rbind("--lua-filter", pkg_resource('lua', c(...))))
}

list_css = function() {
  css = list.files(pkg_resource('css'), '[.]css$', full.names = TRUE)
  setNames(css, gsub('.css$', '', basename(css)))
}

check_css = function(css) {
  valid = names(list_css())
  if (length(invalid <- setdiff(css, valid)) == 0) return()
  invalid = invalid[1]
  maybe = sort(agrep(invalid, valid, value = TRUE))[1]
  hint = if (is.na(maybe)) '' else paste0('; did you mean "', maybe, '"?')
  stop(
    '"', invalid, '" is not a valid built-in CSS filename', if (hint != "") hint else ".",
    " Use `pagedown:::list_css()` to view all built-in CSS filenames.", call. = FALSE
  )
}

merge_list = function(x, y) {
  x[names(y)] = y
  x
}

to_json = function(x, ..., auto_unbox = TRUE, null = 'null') {
  jsonlite::toJSON(x, ..., auto_unbox = auto_unbox, null = null)
}

`%n%` = knitr:::`%n%`

run_servr = function() {
  # see https://github.com/rstudio/httpuv/issues/250
  later::with_loop(later::global_loop(), httpuv::service(NA))
}

#' Create posters in HTML
#'
#' The output format \code{poster_relaxed()} is based on an example in the
#' Github repo \url{https://github.com/RelaxedJS/ReLaXed-examples}. See
#' \url{https://pagedown.rbind.io/poster-relaxed/} for an example.
#' @param ...,css,template,number_sections See \code{\link{html_paged}()}.
#' @return An R Markdown output format.
#' @export
poster_relaxed = function(
  ..., css = 'poster-relaxed', template = pkg_resource('html', 'poster-relaxed.html'),
  number_sections = FALSE
) {
  html_format(
    ..., css = css, template = template, theme = NULL, number_sections = number_sections
  )
}


# TODO: most posters like https://www.overleaf.com/gallery/tagged/poster

#' @description The output format \code{poster_jacobs()} mimics the style of the
#'   \dQuote{Jacobs Landscape Poster LaTeX Template Version 1.0} at
#'   \url{https://www.overleaf.com/gallery/tagged/poster}. See
#'   \url{https://pagedown.rbind.io/poster-jacobs/} for an example.
#' @rdname poster_relaxed
#' @export
poster_jacobs = function(
  ..., css = 'poster-jacobs', template = pkg_resource('html', 'poster-jacobs.html')
) {
  poster_relaxed(..., css = css, template = template)
}

library(testit)

assert('pkg_resource() finds files under the resources dir of the package', {
  (dir.exists(pkg_resource(c('css', 'js'))))

  res = lua_filters('loft.lua', 'footnotes.lua')
  (sum(res == '--lua-filter') == 2)
  (length(res) == 4)
})

assert('check_css() validates CSS file paths', {
  (is.null(check_css(c('default', 'letter'))))
  (has_error(check_css('default2'), silent = TRUE))
})

assert('gen_toc_gs() works', {
  (gen_toc_gs(list()) %==% NULL)

  toc = list(list(title = 'a', page = 3, children = list()))
  res = '[/Count 0 /Title <a> /Page 3 /OUT pdfmark'

  (gen_toc_gs(toc) %==% res)

  toc = list(
    list(title = 'a', page = 3, children = list(
      list(title = 'a-1', page = 5, children = list()),
      list(title = 'a-2', page = 8, children = list(
        list(title = 'a-2-1', page = 9, children = list()),
        list(title = 'a-2-2', page = 10, children = list())
      ))
    )),
    list(title = 'b', page = 20, children = list())
  )
  res = c(
    "[/Count 2 /Title <a> /Page 3 /OUT pdfmark",
    "[/Count 0 /Title <a-1> /Page 5 /OUT pdfmark",
    "[/Count 2 /Title <a-2> /Page 8 /OUT pdfmark",
    "[/Count 0 /Title <a-2-1> /Page 9 /OUT pdfmark",
    "[/Count 0 /Title <a-2-2> /Page 10 /OUT pdfmark",
    "[/Count 0 /Title <b> /Page 20 /OUT pdfmark"
  )

  (gen_toc_gs(toc) %==% res)
})

#' Print a web page to PDF or capture a screenshot using the headless Chrome
#'
#' Print an HTML page to PDF or capture a PNG/JPEG screenshot through the Chrome
#' DevTools Protocol. Google Chrome or Microsoft Edge (or Chromium on Linux)
#' must be installed prior to using this function.
#' @param input A URL or local file path to an HTML page, or a path to a local
#'   file that can be rendered to HTML via \code{rmarkdown::\link[rmarkdown]{render}()}
#'   (e.g., an R Markdown document or an R script). If the \code{input} is to be
#'   rendered via \code{rmarkdown::render()} and you need to pass any arguments
#'   to it, you can pass the whole \code{render()} call to
#'   \code{chrome_print()}, e.g., if you need to use the \code{params} argument:
#'   \code{pagedown::chrome_print(rmarkdown::render('input.Rmd', params =
#'   list(foo = 1:10)))}. This is because \code{render()} returns the HTML file,
#'   which can be passed to \code{chrome_print()}.
#' @param output The output filename. For a local web page \file{foo/bar.html},
#'   the default PDF output is \file{foo/bar.pdf}; for a remote URL
#'   \file{https://www.example.org/foo/bar.html}, the default output will be
#'   \file{bar.pdf} under the current working directory. The same rules apply
#'   for screenshots.
#' @param wait The number of seconds to wait for the page to load before
#'   printing (in certain cases, the page may not be immediately ready for
#'   printing, especially there are JavaScript applications on the page, so you
#'   may need to wait for a longer time).
#' @param browser Path to Google Chrome, Microsoft Edge or Chromium. This
#'   function will try to find it automatically via \code{\link{find_chrome}()}
#'   if the path is not explicitly provided and the environment variable
#'   \code{PAGEDOWN_CHROME} is not set.
#' @param format The output format.
#' @param options A list of page options. See
#'   \code{https://chromedevtools.github.io/devtools-protocol/tot/Page#method-printToPDF}
#'    for the full list of options for PDF output, and
#'   \code{https://chromedevtools.github.io/devtools-protocol/tot/Page#method-captureScreenshot}
#'    for options for screenshots. Note that for PDF output, we have changed the
#'   defaults of \code{printBackground} (\code{TRUE}),
#'   \code{preferCSSPageSize} (\code{TRUE}) and when available
#'   \code{transferMode} (\code{ReturnAsStream}) in this function.
#' @param selector A CSS selector used when capturing a screenshot.
#' @param box_model The CSS box model used when capturing a screenshot.
#' @param scale The scale factor used for screenshot.
#' @param work_dir Name of headless Chrome working directory. If the default
#'   temporary directory doesn't work, you may try to use a subdirectory of your
#'   home directory.
#' @param timeout The number of seconds before canceling the document
#'   generation. Use a larger value if the document takes longer to build.
#' @param extra_args Extra command-line arguments to be passed to Chrome.
#' @param verbose Level of verbosity: \code{0} means no messages; \code{1} means
#'   to print out some auxiliary messages (e.g., parameters for capturing
#'   screenshots); \code{2} (or \code{TRUE}) means all messages, including those
#'   from the Chrome processes and WebSocket connections.
#' @param async Execute \code{chrome_print()} asynchronously? If \code{TRUE},
#'   \code{chrome_print()} returns a \code{\link[promises]{promise}} value (the
#'   \pkg{promises} package has to be installed in this case).
#' @param outline If not \code{FALSE}, \code{chrome_print()} will add the
#'   bookmarks to the generated \code{pdf} file, based on the table of contents
#'   informations. This feature is only available for output formats based on
#'   \code{\link{html_paged}}. It is enabled by default, as long as the
#'   Ghostscript executable can be detected by \code{\link[tools]{find_gs_cmd}}.
#' @param encoding Not used. This argument is required by RStudio IDE.
#' @references
#' \url{https://developer.chrome.com/blog/headless-chrome/}
#' @return Path of the output file (invisibly). If \code{async} is \code{TRUE},
#'   this is a \code{\link[promises]{promise}} value.
#' @export
chrome_print = function(
  input, output = xfun::with_ext(input, format), wait = 2, browser = 'google-chrome',
  format = c('pdf', 'png', 'jpeg'), options = list(), selector = 'body',
  box_model = c('border', 'content', 'margin', 'padding'), scale = 1, work_dir = tempfile(),
  timeout = 30, extra_args = c('--disable-gpu'), verbose = 0, async = FALSE,
  outline = gs_available(), encoding
) {
  is_rstudio_knit =
    !interactive() && !is.na(Sys.getenv('RSTUDIO', NA)) &&
    !missing(encoding) && length(match.call()) == 3
  if (is_rstudio_knit) verbose = 1

  format = match.arg(format)

  if (missing(browser) && is.na(browser <- Sys.getenv('PAGEDOWN_CHROME', NA))) {
    browser = find_chrome()
  } else {
    if (!file.exists(browser)) browser = Sys.which(browser)
  }
  if (!utils::file_test('-x', browser)) stop('The browser is not executable: ', browser)
  if (isTRUE(verbose)) verbose = 2
  if (verbose >= 1) message('Using the browser "', browser, '"')

  # check that work_dir does not exist because it will be deleted at the end
  if (dir.exists(work_dir)) stop('The directory ', work_dir, ' already exists.')
  work_dir = normalizePath(work_dir, mustWork = FALSE)

  # for windows, use the --no-sandbox option
  extra_args = unique(c(
    extra_args, proxy_args(), if (xfun::is_windows()) '--no-sandbox',
    '--headless', '--no-first-run', '--no-default-browser-check', '--hide-scrollbars'
  ))

  debug_port = servr::random_port(NULL)
  log_file = if (getOption('pagedown.chrome.log', FALSE)) {
    sprintf('chrome-stderr-%s.log', format(Sys.time(), "%Y-%m-%d_%H-%M-%S"))
  }
  ps = processx::process$new(browser, c(
    paste0('--remote-debugging-port=', debug_port),
    paste0('--user-data-dir=', work_dir), extra_args
  ), stderr = log_file)
  kill_chrome = function(...) {
    if (verbose >= 1) message('Closing browser')
    if (ps$is_alive()) ps$kill()
    if (verbose >= 1) message('Cleaning browser working directory')
    unlink(work_dir, recursive = TRUE)
  }
  on.exit(kill_chrome(), add = TRUE)

  remote_protocol_ok = is_remote_protocol_ok(debug_port, verbose = verbose)
  stream_pdf_available = isTRUE(attr(remote_protocol_ok, 'stream_pdf_available'))

  if (!remote_protocol_ok)
    stop('A more recent version of Chrome is required. ')

  if (format == 'pdf' && stream_pdf_available)
    options = merge_list(list(transferMode = 'ReturnAsStream'), as.list(options))

  # If !async, use a private event loop to drive the websocket. This is
  # necessary to separate later callbacks relevant to chrome_print, from any
  # other callbacks that have been scheduled before entering chrome_print; the
  # latter must not be invoked while inside of any synchronous function,
  # including chrome_print(async=FALSE).
  #
  # It's also critical that none of the code inside with_temp_loop waits on a
  # promise that originates from outside the with_temp_loop, as it will cause
  # the code inside to hang. And finally, no promise from inside with_temp_loop
  # should escape to the outside either, as with_temp_loop uses a truly "temp"
  # loop--it will be destroyed when with_temp_loop completes.
  #
  # Therefore, if async, it's important NOT to use a private event loop.
  with_temp_loop_maybe <- if (async) identity else later::with_temp_loop

  with_temp_loop_maybe({

    ws = websocket::WebSocket$new(get_entrypoint(debug_port, verbose), autoConnect = FALSE)
    ws$onClose(kill_chrome)
    ws$onError(kill_chrome)
    close_ws = function() {
      if (verbose >= 1) message('Closing websocket connection')
      ws$close()
    }

    svr = NULL # init svr variable
    if (file.exists(input)) {
      is_html = function(x) grepl('[.]html?$', x)
      with_msg_maybe = if (is_rstudio_knit) suppressMessages else identity
      url = if (is_html(input)) input else with_msg_maybe(rmarkdown::render(
        input, envir = parent.frame(), encoding = 'UTF-8'
      ))
      if (!is_html(url)) stop(
        "The file '", url, "' should have the '.html' or '.htm' extension."
      )
      svr = servr::httd(
        dirname(url), daemon = TRUE, browser = FALSE, verbose = verbose >= 1,
        port = servr::random_port(NULL), initpath = httpuv::encodeURIComponent(basename(url))
      )
      stop_server = function(...) {
        if (verbose >= 1) message('Closing local webserver')
        svr$stop_server()
      }
      on.exit(stop_server(), add = TRUE)
      ws$onClose(stop_server)
      ws$onError(stop_server)
      url = svr$url
    } else url = input  # the input is not a local file; assume it is just a URL

    # remove hash/query parameters in url
    if (missing(output) && !file.exists(input))
      output = xfun::with_ext(basename(gsub('[#?].*', '', url)), format)
    output2 = normalizePath(output, mustWork = FALSE)
    # try to remove the file and throw a clear error if it still exists as it may be locked
    if (!suppressWarnings(file.remove(output2)) && xfun::file_exists(output2)) {
      stop(
        "The file '", output, "' cannot be overwritten",
        if (format == "pdf") " (may be locked by a PDF reader?)", "."
      )
    }
    if (!dir.exists(d <- dirname(output2)) && !dir.create(d, recursive = TRUE)) stop(
      'Cannot create the directory for the output file: ', d
    )

    if ((format == 'pdf') && !all(c(missing(selector), missing(box_model), missing(scale))))
      warning('For "pdf" format, arguments `selector`, `box_model` and `scale` are ignored.', call. = FALSE)

    box_model = match.arg(box_model)

    pr = NULL
    res_fun = function(value) {} # default: do nothing
    rej_fun = function(reason) {} # default: do nothing
    if (async) {
      pr_print = promises::promise(function(resolve, reject) {
        res_fun <<- resolve
        rej_fun <<- function(reason) reject(paste('Failed to generate output. Reason:', reason))
      })
      pr_timeout = promises::promise(function(resolve, reject) {
        later::later(
          ~reject(paste('Failed to generate output in', timeout, 'seconds (timeout).')),
          timeout
        )
      })
      pr = promises::promise_race(pr_print, pr_timeout)
      promises::finally(pr, close_ws)
    }

    t0 = Sys.time(); token = new.env(parent = emptyenv())
    on.exit({
      close_ws()
      kill_chrome()
      if (!is.null(svr)) stop_server()
    })
    print_page(ws, url, output2, wait, verbose, token, format, options, selector, box_model, scale, outline, res_fun, rej_fun)

    if (async) {
      on.exit()
      return(pr)
    }

    while (!isTRUE(token$done)) {
      if (!is.null(e <- token$error)) stop('Failed to generate output. Reason: ', e)
      if (as.numeric(difftime(Sys.time(), t0, units = 'secs')) > timeout) stop(
        'Failed to generate output in ', timeout, ' seconds (timeout).'
      )
      later::run_now(); if (!is.null(svr)) run_servr()
    }

    if (is_rstudio_knit) message('\nOutput created: ', basename(output))

    invisible(output)
  })
}

gen_toc_gs = function(toc) {
  to_gs = function(x) {
    stopifnot(setequal(names(x), c('title', 'page', 'children')))
    template = "[/Count %d /Title <%s> /Page %d /OUT pdfmark"
    title = x$title
    page = x$page
    count = length(x$children)
    out = sprintf(template, count, title, page)
    if (any(count > 0L)) {
      children = lapply(x$children, to_gs)
      out = c(out, unlist(children, use.names = FALSE))
    }
    out
  }
  unlist(lapply(toc, to_gs), use.names = FALSE)
}

find_gs = function() {
  gs = tools::find_gs_cmd()
  # according to the doc of tools::find_gs_cmd, gs should always be a string
  unname(gs)
}

gs_available = function() {
  nzchar(find_gs())
}

add_outline = function(input, toc_infos, verbose) {
  gs_content = gen_toc_gs(toc_infos)
  # when TOC doesn't exist, gs_content will be null
  if (is.null(gs_content)) return(invisible(input))
  gs_file = tempfile(); on.exit(unlink(gs_file), add = TRUE)
  writeLines(gs_content, con = gs_file)
  if (!gs_available()) stop(
    'Cannot find GhostScript executable automatically. ',
    "Please pass the full path of the GhostScript executable ",
    "to the environment variable 'R_GSCMD'. ",
    "See ?tools::find_gs_cmd for more details."
  )
  output = tempfile(fileext = '.pdf'); on.exit(unlink(output), add = TRUE)
  input2 = input
  if (!xfun::is_ascii(input2)) {
    # this is needed when input contain non-ASCII characters
    input2 = tempfile(fileext = '.pdf'); on.exit(unlink(input2), add = TRUE)
    file.copy(input, input2)
  }
  args = c('-o', output, '-sDEVICE=pdfwrite', '-dPDFSETTINGS=/prepress', input2, gs_file)
  if (verbose < 2) args = c('-q', args)
  gs_out = system2(find_gs(), shQuote(args))
  if (gs_out == 0) {
    file.copy(output, input, overwrite = TRUE)
  } else {
    warning('GhostScript fails to add the outlines', call. = FALSE)
  }
  invisible(input)
}

#' Find Google Chrome, Microsoft Edge or Chromium in the system
#'
#' On Windows, this function tries to find Chrome or Edge from the registry. On
#' macOS, it returns a hard-coded path of Chrome under \file{/Applications}. On
#' Linux, it searches for \command{chromium-browser} and \command{google-chrome}
#' from the system's \var{PATH} variable.
#' @return A character string.
#' @export
find_chrome = function() {
  switch(
    .Platform$OS.type,
    windows = {
      res = unlist(lapply(c('ChromeHTML', 'MSEdgeHTM'), function(x) {
        res = tryCatch({
          unlist(utils::readRegistry(sprintf('%s\\shell\\open\\command', x), 'HCR'))
        }, error = function(e) '')
        res = unlist(strsplit(res, '"'))
        res = head(res[file.exists(res)], 1)
      }))
      if (length(res) < 1) stop(
        'Cannot find Google Chrome or Edge automatically from the Windows Registry Hive. ',
        "Please pass the full path of chrome.exe or msedge.exe to the 'browser' argument ",
        "or to the environment variable 'PAGEDOWN_CHROME'."
      )
      res[1]
    },
    unix = if (xfun::is_macos()) {
      '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome'
    } else {
      for (i in c('google-chrome', 'chromium-browser', 'chromium', 'google-chrome-stable')) {
        if ((res <- Sys.which(i)) != '') break
      }
      if (res == '') stop('Cannot find Chromium or Google Chrome')
      res
    },
    stop('Your platform is not supported')
  )
}

proxy_args = function() {
  # the order of the variables is important because the first non-empty variable is kept
  val = Sys.getenv(c('https_proxy', 'HTTPS_PROXY', 'http_proxy', 'HTTP_PROXY'))
  val = val[val != '']
  if (length(val) == 0) return()
  c(
    paste0('--proxy-server=', val[1]),
    paste0('--proxy-bypass-list=', paste(no_proxy_urls(), collapse = ';'))
  )
}

no_proxy_urls = function() {
  x = do.call(c, strsplit(Sys.getenv(c('no_proxy', 'NO_PROXY')), '[,;]'))
  x = c('localhost', '127.0.0.1', x)
  unique(x)
}

is_remote_protocol_ok = function(debug_port,
                                 verbose = 0) {
  url = sprintf('http://127.0.0.1:%s/json/protocol', debug_port)
  # can be specify with option, for ex. for CI specificity. see #117
  max_attempts = getOption("pagedown.remote.maxattempts", 20L)
  sleep_time = getOption("pagedown.remote.sleeptime", 0.5)
  if (verbose >= 1) message('Trying to find headless Chrome in ', max_attempts, ' attempts')
  for (i in seq_len(max_attempts)) {
    remote_protocol = tryCatch(suppressWarnings(jsonlite::read_json(url)), error = function(e) NULL)
    if (!is.null(remote_protocol)) {
      if (verbose >= 1) message('Headless Chrome found at attempt ', i)
      break
    }
    if (i == max_attempts) stop('Cannot find headless Chrome after ', max_attempts, ' attempts')
    Sys.sleep(sleep_time)
  }

  required_commands = list(
    DOM = c('enable', 'getBoxModel', 'getDocument', 'querySelector'),
    Network = c('enable'),
    Page = c('addScriptToEvaluateOnNewDocument',
             'captureScreenshot',
             'enable',
             'navigate',
             'printToPDF'
    ),
    Runtime = c('enable', 'addBinding', 'evaluate'),
    Target = c('attachToTarget', 'createTarget')
  )

  remote_domains = sapply(remote_protocol$domains, `[[`, 'domain')
  if (!all(names(required_commands) %in% remote_domains))
    return(FALSE)

  required_events = list(
    Inspector = c('targetCrashed'),
    Network = c('responseReceived'),
    Page = c('loadEventFired'),
    Runtime = c('bindingCalled', 'exceptionThrown')
  )

  remote_commands = sapply(names(required_commands), function(domain) {
    sapply(
      remote_protocol$domains[remote_domains %in% domain][[1]]$commands,
      `[[`, 'name'
    )
  })

  remote_events =  sapply(names(required_events), function(domain) {
    sapply(
      remote_protocol$domains[remote_domains %in% domain][[1]]$events,
      `[[`, 'name'
    )
  })

  if (!all(mapply(function(x, table) all(x %in% table), required_commands, remote_commands),
      mapply(function(x, table) all(x %in% table), required_events, remote_events)
  ))
    return(FALSE)

  stream_pdf_available = 'transferMode' %in% sapply(
    remote_protocol[['domains']][remote_domains == 'Page'][[1]][['commands']][remote_commands[['Page']] == 'printToPDF'][[1]][['parameters']],
    `[[`, 'name'
  )
  res = TRUE
  attr(res, 'stream_pdf_available') = stream_pdf_available

  res
}

get_entrypoint = function(debug_port, verbose) {
  version_infos = jsonlite::read_json(
    sprintf('http://127.0.0.1:%s/json/version', debug_port), simplifyVector = TRUE
  )
  browser = version_infos$webSocketDebuggerUrl
  if (length(browser) == 0) stop("Cannot find 'Browser' websocket URL. Please retry.")
  if (verbose >= 1)
    message('Browser version: ', version_infos$Browser)
  browser
}

print_page = function(
  ws, url, output, wait, verbose, token, format,
  options = list(), selector, box_model, scale, outline, resolve, reject
) {
  # init values
  session_id = NULL
  coords = NULL
  toc_infos = NULL
  stream_handle = NULL
  con = NULL

  ws$onOpen(function(event) {
    # Create a new Target (tab)
    ws$send(to_json(list(
      id = 1, method = 'Target.createTarget',
      params = list(url = 'about:blank')
    )))
  })

  ws$onMessage(function(event) {
    if (!is.null(token$error)) {
      ws$close()
      reject(token$error)
      return()
    }
    if (verbose >= 2) message('Message received from headless Chrome: ', event$data)
    msg = jsonlite::fromJSON(event$data)
    id = msg$id
    method = msg$method

    if (!is.null(token$error <- msg$error$message)) {
      ws$close()
      reject(token$error)
      return()
    }

    if (!is.null(id)) switch(
      id,
      # Command #1 received -> callback: command #2 Target.attachToTarget in flat mode
      ws$send(to_json(list(
        id = 2, method = 'Target.attachToTarget',
        params = list(targetId = msg$result$targetId, flatten = TRUE)
      ))),
      # Command #2 received -> store the sessionId; callback: command #3 Runtime.enable
      {
        session_id <<- msg$result$sessionId
        ws$send(to_json(list(
          id = 3, sessionId = session_id, method = 'Runtime.enable'
        )))
      },
      # Command #3 received -> callback: command #4 Page.enable
      ws$send(to_json(list(
        id = 4, sessionId = session_id, method = 'Page.enable'
      ))),
      # Command #4 received -> callback: command #5 Runtime.addBinding
      ws$send(to_json(list(
        id = 5, sessionId = session_id, method = "Runtime.addBinding",
        params = list(name = "pagedownListener")
      ))),
      # Command #5 received -> callback: command #6 Network.Enable
      ws$send(to_json(list(
        id = 6, sessionId = session_id, method  = 'Network.enable'
      ))),
      # Command #6 received -> callback: command #7 Page.addScriptToEvaluateOnNewDocument
      ws$send(to_json(list(
        id = 7, sessionId = session_id, method = "Page.addScriptToEvaluateOnNewDocument",
        params = list(source = paste0(readLines(pkg_resource('js', 'chrome_print.js')), collapse = ""))
      ))),
      # Command #7 received -> callback: command #8 Page.Navigate
      ws$send(to_json(list(
        id = 8, sessionId = session_id, method= 'Page.navigate',
        params = list(url = url)
      ))),
      {
        # Command #8 received - check if there is an error when navigating to url
        if(!is.null(token$error <- msg$result$errorText)) {
          reject(token$error)
        }
      },
      {
        # Command #9 received - Test if the html document uses the paged.js polyfill
        # if not, call the binding when HTMLWidgets, MathJax and fonts are ready
        # (see inst/resources/js/chrome_print.js)
        if (!isTRUE(msg$result$result$value)) {
          ws$send(to_json(list(
            id = 10, sessionId = session_id, method = "Runtime.evaluate",
            params = list(expression = "pagedownReady.then(() => {pagedownListener('{\"pagedjs\":false}');})")
          )))
        }
      },
      # Command #10 received - No callback
      NULL,
      # Command #11 received -> callback: command #12 DOM.getDocument
      ws$send(to_json(list(id = 12, sessionId = session_id, method = "DOM.getDocument"))),
      # Command #12 received -> callback: command #13 DOM.querySelector
      ws$send(to_json(list(
        id = 13, sessionId = session_id, method = "DOM.querySelector",
        params = list(nodeId = msg$result$root$nodeId, selector = selector)
      ))),
      {
        # Command 13 received -> callback: command #14 DOM.getBoxModel
        if (msg$result$nodeId == 0) {
          token$error <- 'No element in the HTML page corresponds to the `selector` value.'
          reject(token$error)
        } else {
          ws$send(to_json(list(
            id = 14, sessionId = session_id, method = "DOM.getBoxModel",
            params = list(nodeId = msg$result$nodeId)
          )))
        }
      },
      {
        # Command 14 received -> callback: command #15 Emulation.setDeviceMetricsOverride
        coords <<- msg$result$model[[box_model]]
        device_metrics = list(
          width = ceiling(coords[5]),
          height = ceiling(coords[6]),
          deviceScaleFactor = 1,
          mobile = FALSE
        )
        ws$send(to_json(list(
          id = 15, sessionId = session_id, method = 'Emulation.setDeviceMetricsOverride',
          params = device_metrics
        )))
      },
      {
        # Command #15 received -> callback: command #16 Page.captureScreenshot
        opts = as.list(options)

        origin = as.list(coords[1:2])
        names(origin) = c('x', 'y')

        dims = as.list(coords[5:6] - coords[1:2])
        names(dims) = c('width', 'height')

        clip = c(origin, dims, list(scale = scale))
        opts = merge_list(list(clip = clip), opts)
        if (verbose >= 1) message(
          'Screenshot captured with the following value for the `options` parameter:\n',
          paste0(deparse(opts), collapse = '\n ')
        )
        opts$format = format

        ws$send(to_json(list(
          id = 16, sessionId = session_id, method = 'Page.captureScreenshot',
          params = opts
        )))
      },
      {
        # Command #16 received (printToPDF or captureScreenshot)
        # if data are received -> callback: save to file & close Chrome
        # if a stream handle is received -> callback: command #17 IO.read
        if (is.null(stream_handle <<- msg$result$stream)) {
          writeBin(jsonlite::base64_dec(msg$result$data), output)
          if (!isFALSE(outline) && length(toc_infos)) add_outline(output, toc_infos, verbose)
          resolve(output)
          token$done = TRUE
        } else {
          if (verbose >= 1) message('Receiving PDF from a stream')
          # open a connection
          con <<- file(output, 'wb')
          # read the first chunk of the stream
          ws$send(to_json(list(
            id = 17, sessionId = session_id, method = 'IO.read',
            params = list(handle = stream_handle)
          )))
        }
      },
      {
        # Command #17 received
        # if there is another chunk to read -> callback: IO.read
        # if EOF -> callback: command #18 IO.close
        if (verbose >= 1) message('    stream chunk received')
        if (isTRUE(msg$result$base64Encoded)) {
          writeBin(jsonlite::base64_dec(msg$result$data), con)
        } else {
          writeBin(msg$result$data, con)
        }

        if (isTRUE(msg$result$eof)) {
          if (verbose >= 1) message(
            'No more stream chunk to read\n    closing stream'
          )
          close(con)
          ws$send(to_json(list(
            id = 18, sessionId = session_id, method = 'IO.close',
            params = list(handle = stream_handle)
          )))
        } else {
          # read another chunk
          ws$send(to_json(list(
            id = 17, sessionId = session_id, method = 'IO.read',
            params = list(handle = stream_handle)
          )))
        }
      },
      {
        # Command #18 received -> callback: add outline & close Chrome
        if (!isFALSE(outline) && length(toc_infos)) add_outline(output, toc_infos, verbose)
        resolve(output)
        token$done = TRUE
      }
    )
    if (!is.null(method)) {
      if (method == "Network.responseReceived") {
        status = as.numeric(msg$params$response$status)
        # treat a failure of the main document as fatal; warn about sub-resources
        # (favicon.ico, images, fonts, etc.) that fail but don't block rendering
        if (status >= 400) {
          err = sprintf(
            'Failed to open %s (HTTP status code: %s)', msg$params$response$url, status
          )
          if (identical(msg$params$type, "Document")) {
            reject(token$error <- err)
          } else {
            warning(err, call. = FALSE)
          }
        }
      }
      if (method == 'Inspector.targetCrashed') {
        token$error = paste(
          'Chrome crashed.',
          'This may be caused by insufficient resources.',
          'Please, try to add "--disable-dev-shm-usage" to the `extra_args` argument.'
        )
        reject(token$error)
      }
      if (method == 'Runtime.exceptionThrown') {
        warning(
          'A runtime exception has occured while executing JavaScript\n',
          '  Runtime exception message:\n    ',
          msg$params$exceptionDetails$exception$description,
          call. = FALSE, immediate. = TRUE
        )
      }
      if (method == "Page.loadEventFired") {
        ws$send(to_json(list(
          id = 9, sessionId = session_id, method = 'Runtime.evaluate',
          params = list(expression = "!!window.PagedPolyfill")
        )))
      }
      if (method == "Runtime.bindingCalled") {
        Sys.sleep(wait)
        opts = as.list(options)
        payload = jsonlite::fromJSON(msg$params$payload, simplifyVector = FALSE)
        toc_infos <<- payload$tocInfos
        if (verbose >= 1 && payload$pagedjs) {
          message("Rendered ", payload$pages, " pages in ", payload$elapsedtime, " milliseconds.")
        }
        if (format == 'pdf') {
          opts = merge_list(list(printBackground = TRUE, preferCSSPageSize = TRUE), opts)
          ws$send(to_json(list(
            id = 16, sessionId = session_id, params = opts, method = 'Page.printToPDF'
          )))
        } else {
          ws$send(to_json(list(id = 11, sessionId = session_id, method = "DOM.enable")))
        }
      }
    }
  })

  ws$connect()
}

#' Create business cards
#'
#' This output format is based on an example in the Github repo
#' \url{https://github.com/RelaxedJS/ReLaXed-examples}. See
#' \url{https://pagedown.rbind.io/business-card/} for an example.
#'
#' @return An R Markdown output format.
#' @export
#' @examples pagedown::business_card()
#' @inheritParams html_paged
business_card = function(template = pkg_resource('html', 'card.html')) {
  rmarkdown::output_format(
    list(opts_chunk = list(echo = FALSE)),
    rmarkdown::pandoc_options('html', 'markdown', args = c(
      '--template', rmarkdown::pandoc_path_arg(template),
      '--variable', 'pagetitle=Business Card'
    ))
  )
}

library(testit)

is_pdf = function(file) {
  identical(readBin(file, 'raw', 5L), charToRaw('%PDF-'))
}

run_promise = function(x) {
  # There's no easy way to test the promise object so it's based on Romain Lesur's code at
  # https://github.com/RLesur/crrri/blob/48b5c2c5e0d209d3742720d30ed91694a6e80c12/R/hold.R#L34-L66
  state <- new.env()
  state$pending = TRUE
  promises::then(
    x,
    onFulfilled = function(value) {
      state$pending = FALSE
      state$value = value
    },
    onRejected = function(error) {
      state$pending = FALSE
      state$reason = error
    }
  )
  while(state$pending) {
    later::run_now(all = FALSE)
  }
  if (!is.null(state$reason)) stop(state$reason)
  state$value
}

# Extract the PDF outline as a nested list of title/children, comparable across
# pdftools versions. pdftools >= 3.8.0 added an `is_open` field to pdf_toc(),
# which also shifted the entries from `[[2L]]` to the `children` element; we
# therefore select `children` by name and keep only `title`/`children`.
pdf_outline = function(file) {
  prune = function(x) lapply(x, function(node) list(
    title = node$title, children = prune(node$children)
  ))
  prune(pdftools::pdf_toc(file)$children)
}

assert('find_chrome() finds Chrome executable', {
  (nzchar(find_chrome()))
})

assert('chrome_print() works with an url', {
  (is_pdf(print_pdf('https://www.r-project.org')))
})

assert('chrome_print() works with a local file path', {
  r_faq_html = file.path(R.home('doc'), 'manual', 'R-FAQ.html')

  (is_pdf(print_pdf(r_faq_html)))
})

assert('chrome_print() supports PDF streaming', {
  r_faq_html = file.path(R.home('doc'), 'manual', 'R-FAQ.html')

  (is_pdf(print_pdf(r_faq_html, options = list(transferMode = 'ReturnAsStream'))))
})


assert('chrome_print() works with html_paged format', {
  (is_pdf(print_pdf('test-chrome.Rmd')))
})

assert('chrome_print() works with reveal.js presentations', {
  f = print_pdf('test-revealjs.Rmd')

  (is_pdf(f))

  # in theory it should be 5 pages; I don't know why it's 6 instead
  (pdftools::pdf_info(f)$pages %==% 7L)

  (pdftools::pdf_text(f)[1] %==% 'Test reveal.js\n')
})

assert('find_gs() finds Ghostscript executable', {
  (nzchar(find_gs()))
  (gs_available())
})

assert('chrome_print() generates expected outline', {
  f = print_pdf('test-outline.Rmd')

  (is_pdf(f))

  toc = pdf_outline(f)
  res = list(
    list(title = "1 Title 1", children = list()),
    list(
      title = "2 Title 2",
      children = list(
        list(title = "2.1 Title 2-1", children = list()),
        list(title = "2.2 Title 2-2", children = list(
          list(title = "2.2.1 Title 2-2-1", children = list()),
          list(title = "2.2.2 Title 2-2-2",children = list())
        ))
      )
    ),
    list(title = "3 \u4e2d\u6587", children = list()),
    list(title = "4 \u4e2d\u6587 \u5e26\u7a7a\u683c", children = list(
      list(title = "4.1 \u4e2d\u6587\u6b21\u7ea7\u6807\u9898", children = list())
    ))
  )

  (toc %==% res)

  # works for async
  f = run_promise(print_pdf('test-outline.Rmd', async = TRUE))
  (is_pdf(f))
  toc = pdf_outline(f)
  (toc %==% res)

  # works for output name with non-ASCII & white space
  output = tempfile(pattern = "\u4e2d \u6587")
  f = print_pdf('test-outline.Rmd', output = output)
  (is_pdf(f))
  toc = pdf_outline(f)
  (toc %==% res)

  # works for input name with non-ASCII & white space
  input = tempfile(pattern = "\u4e2d \u6587", fileext = '.Rmd')
  file.copy('test-outline.Rmd', input)
  f = print_pdf(input)
  (is_pdf(f))
  toc = pdf_outline(f)
  (toc %==% res)
})

# run tests on CI (these tests depend on Chrome)

print_pdf = function(input, output = tempfile(), ...) {
  pagedown::chrome_print(input, output, ...)
}

if (!is.na(Sys.getenv('CI', NA))) {
  options(pagedown.remote.maxattempts = 100L)
  testit::test_pkg('pagedown', 'test-ci')
}

#' Create a paged HTML document suitable for printing
#'
#' This is an output format based on \code{bookdown::html_document2} (which
#' means you can use those Markdown features added by \pkg{bookdown}). The HTML
#' output document is split into multiple pages via a JavaScript library
#' \pkg{paged.js}. These pages contain elements commonly seen in PDF documents,
#' such as page numbers and running headers.
#'
#' When a path or an url is passed to the \code{front_cover} or \code{back_cover}
#' argument, several CSS variables are created. They are named \code{--front-cover}
#' and \code{--back-cover} and can be used as value for the CSS property \code{background-image}.
#' For example, \code{background-image: var(--front-cover);}. When a vector of
#' paths or urls is used as a value for \code{front_cover} or \code{back_cover},
#' the CSS variables are suffixed with an index: \code{--front-cover-1},
#' \code{--front-cover-2}, etc.
#'
#' @param ... Arguments passed to
#'   \code{bookdown::\link[bookdown]{html_document2}}.
#' @param css A character vector of CSS and Sass file paths. If a path
#'   does not contain the \file{.css}, \file{.sass}, or \file{.scss} extension,
#'   it is assumed to be a built-in CSS file. For example, \code{default-fonts}
#'   means the file\code{pagedown:::pkg_resource('css', 'default-fonts.css')}.
#'   To see all built-in CSS files, run \code{pagedown:::list_css()}.
#' @param theme The Bootstrap theme. By default, Bootstrap is not used.
#' @param template The path to the Pandoc template to convert Markdown to HTML.
#' @param csl The path of the Citation Style Language (CSL) file used to format
#'   citations and references (see the \href{https://pandoc.org/MANUAL.html#citations}{Pandoc documentation}).
#' @param front_cover,back_cover Paths or urls to image files to be used
#'   as front or back covers. Theses images are available through CSS variables
#'   (see Details).
#' @references \url{https://pagedown.rbind.io}
#' @return An R Markdown output format.
#' @import stats utils
#' @export
html_paged = function(
  ..., css = c('default-fonts', 'default-page', 'default'), theme = NULL,
  template = pkg_resource('html', 'paged.html'), csl = NULL,
  front_cover = NULL, back_cover = NULL
) {
  html_format(
    ..., css = css, theme = theme, template = template, .pagedjs = TRUE,
    .pandoc_args = c(
      lua_filters('uri-to-fn.lua', 'loft.lua', 'footnotes.lua'), # uri-to-fn.lua must come before footnotes.lua
      if (!is.null(csl)) c('--csl', csl),
      pandoc_chapter_name_args(),
      pandoc_covers_args(front_cover, back_cover)
    ),
    .dependencies = covers_dependencies(front_cover, back_cover)
  )
}

#' Create a letter in HTML
#'
#' This output format is similar to \code{html_paged}. The only differences are
#' in the default stylesheets and the default value of the \code{fig_caption}
#' parameter which is set to \code{FALSE}. See
#' \url{https://pagedown.rbind.io/html-letter/} for an example.
#' @param ...,css,fig_caption Arguments passed to \code{\link{html_paged}()}.
#' @return An R Markdown output format.
#' @export
html_letter = function(..., css = c('default', 'letter'), fig_caption = FALSE) {
  html_paged(..., css = css, fig_caption = fig_caption)
}

#' Create a book for Chapman & Hall/CRC
#'
#' This output format is similar to \code{\link{html_paged}}. The only
#' difference is in the default stylesheets.
#' @param ...,css Arguments passed to \code{\link{html_paged}()}.
#' @return An R Markdown output format.
#' @export
book_crc = function(..., css = c('crc-page', 'default-page', 'default', 'crc')) {
  # see https://github.com/rstudio/pagedown/issues/41 that explains why we need a specific crc-page.css file
  html_paged(..., css = css)
}

#' Create an article for the Journal of Statistical Software
#'
#' This output format is similar to \code{\link{html_paged}}.
#' @param ...,css,template,csl,highlight,pandoc_args Arguments passed to \code{\link{html_paged}()}.
#' @return An R Markdown output format.
#' @export
jss_paged = function(
  ..., css = c('jss-fonts', 'jss-page', 'jss'),
  template = pkg_resource('html', 'jss_paged.html'),
  csl = pkg_resource('csl', 'journal-of-statistical-software.csl'),
  highlight = NULL, pandoc_args = NULL
) {
  jss_format = html_paged(
    ..., template = template, css = css,
    csl = csl, highlight = highlight,
    number_sections = FALSE,
    pandoc_args = c(
      lua_filters('jss.lua'),
      '--metadata', 'link-citations=true',
      pandoc_args
    )
  )

  opts_jss = list(
    prompt = TRUE, comment = NA, R.options = list(prompt = 'R> ', continue = 'R+ '),
    fig.align = 'center', fig.width = 4.9, fig.height = 3.675,
    class.source = 'r-chunk-code'
  )
  for (i in names(opts_jss)) {
    jss_format$knitr$opts_chunk[[i]] = opts_jss[[i]]
  }

  jss_format
}

#' Create a paged HTML thesis document suitable for printing
#'
#' This output format is similar to \code{\link{html_paged}}. The only
#' difference is in the default stylesheets and Pandoc template. See
#' \url{https://pagedown.rbind.io/thesis-paged/} for an example.
#' @param ...,css,template Arguments passed to \code{\link{html_paged}()}.
#' @return An R Markdown output format.
#' @export
thesis_paged = function(
  ..., css = c('thesis'), template = pkg_resource('html', 'thesis.html')
) {
  html_paged(..., css = css, template = template)
}

pagedown_dependency = function(css = NULL, js = FALSE, .test = FALSE) {
  paged = if (.test) 'js/paged-latest.js' else c('js/paged.js', 'js/hooks.js')
  list(htmltools::htmlDependency(
    'paged', packageVersion('pagedown'), src = pkg_resource(),
    script = if (js) c('js/config.js', paged),
    stylesheet = file.path('css', css), all_files = .test
  ))
}

html_format = function(
  ..., self_contained = TRUE, anchor_sections = FALSE, mathjax = 'default', css, template, pandoc_args = NULL,
  .dependencies = NULL, .pagedjs = FALSE, .pandoc_args = NULL, .test = FALSE
) {
  if (!identical(mathjax, 'local')) {
    if (identical(mathjax, 'default'))
      mathjax = rmarkdown:::default_mathjax()

    # workaround the rmarkdown warning when self_contained is TRUE
    # see https://github.com/rstudio/pagedown/issues/128#issuecomment-518371613
    if (isTRUE(self_contained) && !is.null(mathjax)) {
      pandoc_args = c(pandoc_args, paste0('--mathjax=', mathjax))
      mathjax = NULL # let rmarkdown believe that we do not use MathJax
    }
  }

  css2 = grep('[.](?:sa|sc|c)ss$', css, value = TRUE, invert = TRUE)
  css  = setdiff(css, css2)
  check_css(css2)

  pandoc_args = c(
    .pandoc_args,
    pandoc_args,
    # use the pagebreak pandoc filter provided by rmarkdown 1.16:
    if (isTRUE(.pagedjs)) pandoc_metadata_arg('newpage_html_class', 'page-break-after')
  )

  html_document2 = function(..., extra_dependencies = list()) {
    bookdown::html_document2(..., extra_dependencies = c(
      extra_dependencies, .dependencies,
      pagedown_dependency(xfun::with_ext(css2, '.css'), .pagedjs, .test)
    ))
  }

  fmt = html_document2(
    ..., self_contained = self_contained, anchor_sections = anchor_sections, mathjax = mathjax, css = css,
    template = template, pandoc_args = pandoc_args
  )


  if (isTRUE(.pagedjs)) {
    # shadow DOM needs to be deactivated (until https://gitlab.coko.foundation/pagedjs/pagedjs/-/issues/148 is solved)
    # e.g this is required for the flextable package (see https://github.com/rstudio/pagedown/issues/216) so we need to pass
    # information about paged.js being used. 
    fmt$knitr$opts_knit[['is.paged.js']] = TRUE

    # Keeping this for flextable < 0.8.6
    fmt$knitr$opts_chunk[['ft.shadow']] = FALSE
  }

  fmt
}

chapter_name = function() {
  config = bookdown:::load_config()
  chapter_name = config[['chapter_name']] %n% bookdown:::ui_language('chapter_name')
  if (is.null(chapter_name) || identical(chapter_name, '')) return()
  if (!is.character(chapter_name)) stop(
    'chapter_name in _bookdown.yml must be a character string'
  )
  if (length(chapter_name) > 2) stop('chapter_name must be of length 1 or 2')
  chapter_name
}

pandoc_metadata_arg = function(name, value) {
  if (!missing(value) && is.character(value)) {
    value = deparse(value)
  }
  c('--metadata', if (missing(value)) name else paste0(name, '=', value))
}

pandoc_chapter_name_args = function() {
  unlist(lapply(chapter_name(), pandoc_metadata_arg, name = 'chapter_name'))
}

pandoc_covers_args = function(front_cover, back_cover) {
  build_links = function(name, img) {
    if (length(img) == 0) return()
    name = paste(name, seq_along(img), sep = '-')
    links = paste0('<link id="', name,'-pagedown-attachment" rel="attachment" href="', img, '" />')
    links[!file.exists(img)]
  }
  links = c(build_links('front-cover', front_cover), build_links('back-cover', back_cover))
  if (length(links)) {
    writeLines(links, f <- tempfile(fileext = ".html"))
    rmarkdown::includes_to_pandoc_args(rmarkdown::includes(f))
  }
}

covers_dependencies = function(front_cover, back_cover) {
  html_dep = function(name, img) {
    htmltools::htmlDependency(
      name, packageVersion('pagedown'), dirname(path.expand(img)),
      attachment = c(pagedown = basename(img)), all_files = FALSE
    )
  }

  build_deps = function(name, img) {
    if (length(img) == 0) return(list())
    name = paste(name, seq_along(img), sep = '-')
    name = name[file.exists(img)]
    img = img[file.exists(img)]
    mapply(name, img, FUN = html_dep, USE.NAMES = FALSE, SIMPLIFY = FALSE)
  }

  c(build_deps('front-cover', front_cover), build_deps('back-cover', back_cover))
}

---
title: "Unnecessarily Complicated Research Title"
author: "John Smith^1^, James Smith^1,2^, and Jane Smith^3^"
institute: "1. Department and University One; 2. Department and University Two; 3. Department and University Three"
date: "2018-11-30"
references:
  - id: R-base
    author:
    - family: "R Core Team"
      given: ""
    title: 'R: A Language and Environment for Statistical Computing'
    issued:
      year: 2018
    URL: https://www.r-project.org
  - id: R-pagedown
    author:
    - family: Xie
      given: Yihui
    - family: Lesur
      given: Romain
    title: 'Paginate the HTML Output of R Markdown with CSS for Print'
    issued:
      year: 2018
    URL: https://github.com/rstudio/pagedown
output:
  pagedown::poster_jacobs:
    self_contained: false
    pandoc_args: --mathjax
---

Objectives {.block}
================================================================================

Lorem ipsum dolor sit amet, consectetur, nunc tellus pulvinar tortor, commodo eleifend risus arcu sed odio:

- Mollis dignissim, magna augue tincidunt dolor, interdum vestibulum urna
- Sed aliquet luctus lectus, eget aliquet leo ullamcorper consequat. Vivamus eros sem, iaculis ut euismod non, sollicitudin vel orci.
- Nascetur ridiculus mus.  
- Euismod non erat. Nam ultricies pellentesque nunc, ultrices volutpat nisl ultrices a.
- Euismod non erat. Nam ultricies pellentesque nunc, ultrices volutpat nisl ultrices a.
- Euismod non erat. Nam ultricies pellentesque nunc, ultrices volutpat nisl ultrices a.
- Euismod non erat. Nam ultricies pellentesque nunc, ultrices volutpat nisl ultrices a.



Introduction
================================================================================

Lorem ipsum dolor **sit amet**, consectetur adipiscing elit. Donec ut volutpat elit. Sed laoreet accumsan mattis. Integer sapien tellus, auctor ac blandit eget, sollicitudin vitae lorem. Praesent dictum tempor pulvinar. Suspendisse potenti. Sed tincidunt varius ipsum, et porta nulla suscipit et. Etiam congue bibendum felis, ac dictum augue cursus a. **Donec** magna eros, iaculis sit amet placerat quis, laoreet id est.

```{r, cars-plot, fig.width=6, fig.height=3, dev='svg', out.width='100%', fig.cap='Figure Caption.', echo=FALSE}
par(mar = c(4, 4, .1, .1), las = 1)
boxplot(
  Sepal.Length ~ Species, data = iris, horizontal = TRUE,
  col = c('limegreen', 'steelblue', 'orange'), notch = TRUE
)
```

Test citations @R-base @R-pagedown. See Figure \@ref(fig:cars-plot).




Materials
================================================================================

The following materials were required to complete the research:


- Curabitur pellentesque dignissim
- Eu facilisis est tempus quis
- Duis porta consequat lorem
- Eu facilisis est tempus quis


The materials were prepared according to the steps outlined below:

1. Curabitur pellentesque dignissim
1. Eu facilisis est tempus quis
1. Duis porta consequat lorem
1. Curabitur pellentesque dignissim




Methods
================================================================================

Lorem ipsum dolor **sit amet**, consectetur adipiscing elit. Sed laoreet accumsan mattis. Integer sapien tellus, auctor ac blandit eget, sollicitudin vitae lorem. Praesent dictum tempor pulvinar. Suspendisse potenti. Sed tincidunt varius ipsum, et porta nulla suscipit et. Etiam congue bibendum felis, ac dictum augue cursus a. **Donec** magna eros, iaculis sit amet placerat quis, laoreet id est. In ut orci purus, interdum ornare nibh. Pellentesque pulvinar, nibh ac malesuada accumsan, urna nunc convallis tortor, ac vehicula nulla tellus eget nulla. Nullam lectus tortor, _consequat tempor hendrerit_ quis, vestibulum in diam. Maecenas sed diam augue.

Important Result {.block}
================================================================================

Lorem ipsum dolor **sit amet**, consectetur adipiscing elit. Sed commodo molestie porta. Sed ultrices scelerisque sapien ac commodo. Donec ut volutpat elit.



Mathematical Section
================================================================================

Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin. Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin.
  
\begin{equation}
E = mc^{2}
(\#eq:Einstein)
\end{equation}

Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin. Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin.

\begin{equation}
\cos^3 \theta =\frac{1}{4}\cos\theta+\frac{3}{4}\cos 3\theta
(\#eq:refname)
\end{equation}

Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin. Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin.

\begin{equation}
\kappa =\frac{\xi}{E_{\mathrm{max}}}
\end{equation}

Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin. Nam quis odio enim, in molestie libero. Vivamus cursus mi at nulla elementum sollicitudin.


Results
================================================================================

::: {.figure-example}
Placeholder  
~~~~~~~  
Image

Figure 2: Figure caption.
:::

Nunc tempus venenatis facilisis. Curabitur suscipit consequat eros non porttitor. Sed a massa dolor, id ornare enim:

```{r echo=FALSE}
knitr::kable(iris[1:5, 1:4], caption = 'Table caption.')
```


Conclusion
================================================================================

Nunc tempus venenatis facilisis. **Curabitur suscipit** consequat eros non porttitor. Sed a massa dolor, id ornare enim. Fusce quis massa dictum tortor **tincidunt mattis**. Donec quam est, lobortis quis pretium at, laoreet scelerisque lacus. Nam quis odio enim, in molestie libero. Vivamus cursus mi at _nulla elementum sollicitudin_.

Nunc tempus venenatis facilisis. **Curabitur suscipit** consequat eros non porttitor. Sed a massa dolor, id ornare enim. Fusce quis massa dictum tortor **tincidunt mattis**. Donec quam est, lobortis quis pretium at, laoreet scelerisque lacus. Nam quis odio enim, in molestie libero. Vivamus cursus mi at _nulla elementum sollicitudin_.




References
================================================================================

<div id="refs"></div>




Acknowledgements {data-color=red}
================================================================================

Nam mollis tristique neque eu luctus. Suspendisse rutrum congue nisi sed convallis. Aenean id neque dolor. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas.

::: {.logo-example}
Placeholder Logo

Placeholder Logo

Placeholder Logo
:::

Nam mollis tristique neque eu luctus. Suspendisse rutrum congue nisi sed convallis. Aenean id neque dolor. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas.


Contact Information {.block data-color=black data-border-color=orange data-background-color=white}
================================================================================

- Web: https://www.example.org/smithlab
- Email: john@example.org
- Phone: +1 (000) 111 1111

---
title: "Lijia Yu's resume"
author: Lijia Yu
date: "`r Sys.Date()`"
output:
  pagedown::html_resume:
    # set it to true for a self-contained HTML page but it'll take longer to render
    self_contained: false
# uncomment this line to produce HTML and PDF in RStudio:
#knit: pagedown::chrome_print
---

Aside
================================================================================


![Lijia Yu](https://avatars1.githubusercontent.com/u/895125?s=400&v=4){width=80%}


Contact Info {#contact}
--------------------------------------------------------------------------------

- <i class="fa fa-envelope"></i> lijia.yu@outlook.com
- <i class="fa fa-github"></i> [github.com/yulijia](https://github.com/yulijia)
- <i class="fa fa-phone"></i> +1 000-000-0000
- For more information, please contact me via email.


Skills {#skills}
--------------------------------------------------------------------------------

- Experienced in statistical analysis, statistical learning models, and optimization methods.

- Full experience with next generation sequencing data analysis.

- Highly skilled in R, Bash, Perl, Python, LaTeX


Disclaimer {#disclaimer}
--------------------------------------------------------------------------------

This resume was made with the R package [**pagedown**](https://github.com/rstudio/pagedown).

Last updated on `r Sys.Date()`.



Main
================================================================================

Lijia Yu {#title}
--------------------------------------------------------------------------------

### Currently searching for a PhD student position

Please note that this is a *real* resume, and I'm really looking for a PhD
student position at the moment. I made this resume because Yihui asked me
if I'd like to test the **pagedown** package with my resume. If you are
interested in my background and skills, please feel free to contact me.


Education {data-icon=graduation-cap data-concise=true}
--------------------------------------------------------------------------------

### Beijing University of Chemical Technology

B.S. in Information and Computing Sciences

Beijing, China

2010

Thesis: Dyadic wavelet and its application in edge detection

### University of Chinese Academy of Sciences

M.S. in Bioinformatics

Beijing, China

2014

Thesis: A multi-omics study for intra-individual divergence of the distributions between mRNA isoforms in mammals


Research Experience {data-icon=laptop}
--------------------------------------------------------------------------------

### Graduate Research Assistant

Beijing Institute of Genomics, Chinese Academy of Sciences

Beijing, China

2011 - 2014

- Performed computational biology research towards understanding regulation of alternative splicing in human and mouse transcriptome.
- Found EGFR pathway related mutations, aimed to understand the impacts of cancer mutations on EGFR signaling pathway.

### Bioinformatican

My Health Gene Technology Co., Ltd.

Beijing, China

2015 - 2016

- Investigated how cancer cells spread to other parts of the body at the single cell level.

### Visiting Scientist

University of Alabama at Birmingham

AL, USA

2016 - 2018

- Investigated the role of mitochondria in development of cancer.
- Investigated the evolution of genome architecture and its role in important evolutionary events.
- Detected thrombotic thrombocytopenic purpura related mutations in mutiple patients' blood genome.
  

Professional Experience {data-icon=suitcase}
--------------------------------------------------------------------------------

### Data Scientist, intern

SupStat Inc.

Beijing, China

2014

::: concise
- Taught R language to beginners. 
- Wrote Shiny app demos.
- Converted statistical tutorials from SPSS to R language.
:::

### Bioinformatician

My Health Gene Technology Co., Ltd.

Beijing, China

2015 - 2016

::: concise
- Analyzed whole-exome sequencing data. 
- Wrote analysis pipelines of ChIP-seq, single cell DNA-seq and single cell RNA-seq.
- Studied tumor metastasis and wrote research reports. 
- Also did case studies to identify the genetic defect causing rare disease.
:::


Teaching Experience {data-icon=chalkboard-teacher}
--------------------------------------------------------------------------------

### Introduction to R Language for Beginners.

Instructor of R and Data Mining Training Courses at SupStat Inc.

Beijing, China

2014

### Computational Biology and Bioinformatics.

Teaching assistant of GBS CB2-201 courses at UAB

AL, USA

2016 - 2017


Selected Publications and Posters {data-icon=file}
--------------------------------------------------------------------------------

### Genetic and epigenetic signals are found predictive to the distribution of intra-individual divergence of alternative splicing.

Poster for 2013 International Conference of Genomics

Qingdao, China

2014

**Yu L**, Chen B, Zhang Z.

### ESCRT-0 complex modulates Rbf mutant cell survival by regulating Rhomboid endosomal trafficking and EGFR signaling.

J Cell Sci. 2016 May 15;129(10):2075-84.

N/A

2016 

Sheng Z, **Yu L**, Zhang T, Pei X, Li X, Zhang Z and Du W. 

---
title: "An HTML poster example"
author: "Yihui Xie"
date: "`r Sys.Date()`"
output:
  pagedown::poster_relaxed:
    self_contained: false
bibliography: packages.bib
references:
  - id: remarkjs
    author:
    - family: Bang
      given: Ole Petter
    title: 'A simple, in-browser, markdown-driven slideshow tool'
    issued:
      year: 2018
    URL: https://remarkjs.com
  - id: naruto
    author:
    - family: Kishimoto
      given: Masashi
    title: 'Naruto Shippūden'
    issued:
      year: 2007
    URL: https://en.wikipedia.org/wiki/Naruto
---




Presentation Ninja [with]{.tiny} R and xaringan
================================================================================

In late 2016, Yihui discovered remark.js [@remarkjs] and loved it at the first sight. A few weeks later in the R Markdown ecosystem [@R-rmarkdown], an R package was born and named **xaringan** [@R-xaringan], which nobody knows how to pronounce (including Yihui himself, because it was adapted from the Japanese manga series Naruto by @naruto). Anyway, this package has gained some popularity, and some CSS ninja have started contributing themes to it. One day, Yihui was thinking about creating a gallery for existing themes in **xaringan**. After a few replies in the [Github issue](https://github.com/yihui/xaringan/issues/172), he realized there might be enough topics on **xaringan** for a short book. Accidentally, he invented a new development model for writing books: the Github-issue-driven book development.




[Authors]{.red}
================================================================================

We are a team of shinobi and kunoichi who wish to share the fun and secrets of the **xaringan** package with you.

::: member-cards
## Emi Tanaka

![Emi](https://avatars3.githubusercontent.com/u/7620319?s=400&v=4)

Lead author, and the ninja theme author

Emi laid out the first sketch of the book, which made Yihui believe that the book had been half-done.

## Joseph Casillas

![](https://avatars1.githubusercontent.com/u/1747068?s=400&v=4)

Contributor of **xaringan**

"Count me in," replied Joseph when Yihui asked who wanted to co-author the book.

## Eric Nantz

![](https://avatars0.githubusercontent.com/u/1043111?s=400&v=4)

Host of the R Podcast

Yihui is eager to know how much Eric's writing is better than his magnetic voice.

## Yihui Xie

![](https://avatars0.githubusercontent.com/u/163582?s=400&v=4)

Main author of **xaringan**

Yihui knows a bit about R/HTML/CSS/JS and wrote **knitr** [@R-knitr], which became a cornerstone of R Markdown.
:::

[All pictures above are from the authors' Github profiles. This poster was created via the R package [**pagedown**](https://github.com/rstudio/pagedown).]{.disclaimer}




[Motivation]{.blue}
================================================================================

```{r, include=FALSE}
lorem = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, qui officia deserunt mollit anim id est laborum.'
```

## There are many secrets about **xaringan**, **knitr**, and R Markdown to be revealed

`r lorem`

![The **xaringan** logo, adapted from Sharingan of the Uchiha clan.](https://user-images.githubusercontent.com/163582/45438104-ea200600-b67b-11e8-80fa-d9f2a99a03b0.png)

## HTML/CSS/JS are fun to play with

`r paste('-', strwrap(lorem, 230), collapse = '\n')`




[Schedule]{.pink}
================================================================================

```{css, echo=FALSE}
.section-4 {
  background-image: url(https://upload.wikimedia.org/wikipedia/commons/7/7e/Mudra-Naruto-KageBunshin.svg) !important;
  background-size: 40% !important;
  background-position: right top !important;
  background-repeat: no-repeat !important;
}
```


## Outline (2018-12-15)

`r lorem`

## Content (2019-03-01)

`r lorem`

## Review and revision (2019-03-31)

`r lorem`

## Copyediting (2019-04-20)

`r lorem`

## Publishing (2019-05-30)

`r lorem`




[Contents]{.green}
================================================================================

`r lorem`

`r lorem`

```r
# some nice R code here
1 + 1
fit = lm(dist ~ speed, cars)
```
`r lorem`

![The most well-known feature of **xaringan**: the random Moustache Karl (aka `yolo = TRUE`).](https://github.com/yihui/xaringan/releases/download/v0.0.2/karl-moustache.jpg)

`r lorem`

$$\bar{X}=\frac{1}{n}\sum_{i=1}^nX_i$$

`r lorem`




[Bibliography]{.yellow}
================================================================================

```{r, include=FALSE}
knitr::write_bib(c('knitr', 'rmarkdown', 'xaringan'), 'packages.bib')
```

---
phone: "+1 844-448-1212"
email: "info@posit.co"
url: www.posit.co
address: |
  @rstudio
logo: "https://posit.co/wp-content/themes/Posit/dist/images/site-share-image.jpg"
person: 
  - name: Tareef Kawaf
    title: President of Posit Software, PBC
    repeat: 12
  - name: Yihui Xie
    title: Responsible for `r head(grep('(?<!shut)down$', unique(.packages(T)), value=T, perl=T), 2)`
    email: xie@yihui.name
    url: https://yihui.name
    repeat: 12
# googlefonts: "Righteous" # feel free to try other fonts at https://fonts.google.com
paperwidth: 8.5in
paperheight: 11in
cols: 4
rows: 3
output: pagedown::business_card
---

<!-- if you prefer you can create cards in markdown instead of YAML declaration -->

::: {.wrapper data-repeat="12"}
[Another Name]{slot="name"}
[Another Title]{slot="title"}
[someone@posit.co]{slot="email"}
:::


<!-- if you prefer black text on white background, set eval=TRUE on the chunk below -->

```{css, eval=FALSE}
.wrapper {
  color: black;
  background-color: white;
  border: 1px dotted black;
}
.coordinates {
  color: #333;
}
```

---
title: A paged html thesis template for R Markdown users
subtitle: A {pagedown} template.
author: 
  - name: William Brent Thorne
    edu: BSc
degree: Master of Documentation
institute: Typeset University
faculty: Reproducibility and FOSS
department: Templates
location: St. Catharines, ON 
date:
  - year: 2019
    month: August
sign_page: true
dedication: A dedication line or two goes here.
abstract: This is the abstract.
preface: A preface to the thesis.
acknowledge: Put the ackknowledgements here.
committee:
  - name: Jane Doe III
    prefix: Dr
    position: Chair of Department
  - name: John Smith
    prefix: Dr
    position: Faculty Advisor
lof: true
lot: true
toc-title: Contents
output:
  pagedown::thesis_paged: 
    toc: true
    number_sections: yes
    pandoc_args: --mathjax
    self_contained: no
bibliography: packages.bib
link-citations: yes
# uncomment this line to produce HTML and PDF in RStudio:
#knit: pagedown::chrome_print
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE,
                      fig.align = 'center')
```

# Literature Review {-}

This is where you can include a lit review if you don't wish for it to be an individual chapter or to be numbered. To make sure that a section heading is not numbered use the `{-}` notation beside the header text like this:

_**rmarkdown**_:

```markdown
# Literature Review {-}

This is where you can include a lit review if you don't wish for it to be an individual chapter or to be numbered. To make sure that a section heading is not numbered use the `{-}` notation beside the header text like this:
```

# The Basics {.chapter}

## Introduction

This template is based on the `pagedown::html_paged` template and modified to meet the requirements of a generic thesis document. Standard RMarkdown formatting can be used for smooth and distraction free writting, for example I will add a citation for the {knitr} package which is located in the `Thesis.bib` file auto-generated in this template [@R-knitr].

Thanks to the [help](https://github.com/rstudio/pagedown/issues/101) of [Romain Lesur](https://github.com/RLesur) this template has the ability to tag section headers with the word "Chapter ". To have your chapters display as this one (_Chapter 1 The Basics_) use the `{.chapter}` class like this:

_**rmarkdown**_
```markdown
---
output:
  pagedown::thesis_paged
    number_Sections: yes
---
# The Basics {.chapter}

## Introduction

This template is based on the `pagedown::html_paged` template...
```

## Second level heading

Here is some code and a plot, with a figure caption, Figure \@ref(fig:code):

```{r code, fig.cap='The automatic numbering of this figure will only work if it includes a figure caption, user beware!'}
plot(iris$Sepal.Length,
     iris$Sepal.Width,
     col = iris$Species,
     pch = 19)
```

Here is an example of an abbreviation where I use the `<abbr>` html tag <abbr title="Brent Thorne">BT</abbr>. In the future there may be a pandoc solution to abbreviation management, however <abbr title="Hyper Text Markup Language">HTML</abbr> is the way to go for now.

# Understanding the method to the madness {.chapter}

Need formulas? Here's some `mathjax` notation:

$$
\beta = \sum^{1 - k}\frac{\delta D}{\sum\frac{\Delta \gamma A}{X-i}}
$$

## Adding Tables

You can easily add tables to this document like so, and you can also reference them like a figure, Table \@ref(tab:atable):

```{r atable}
knitr::kable(
  iris[1:4, 1:5],
  align = 'c',
  caption = 'A caption for the table.',
  col.names = c("Sepal L", "Sepal W", "Petal L", "Petal W", "Species")
)
```

# The final chapter to my pretend thesis {.chapter}

There are still a few features that would be nice to implement in the future for this template. For instance:

1. It would be great to have the ability to use the [{redoc}](https://github.com/noamross/redoc) package to help those who have the ever dreaded "USE WORD ONLY" supervisors.

2. It would also be great to have the ability to generate a _Reference_ section for each _Chapter_ for those who are writing in the "integrated article" format.

3. Along with that, it would be great to pull the YAML data from a child .Rmd file and use it as a way to add the Chapter titles and any other subsequent information (for example a thesis chapter can often actually be a full manuscript and or published paper which would need to have all authors listed for that chapter specifically as well as the journal publication information/ citation data).

# References {-}

```{r, include=FALSE}
knitr::write_bib(c('knitr', 'rmarkdown', 'xaringan'), 'packages.bib')
```

<!-- This '<div id="refs"></div>' needs to 
be here if you have appendix pages 
otherwise you can remove it.-->
<div id="refs"></div> 

# APPENDIX I {-}

```{r}
knitr::kable(iris[1:10,1:5], align = 'c')
```

---
title: "A Multi-page HTML Document"
author: "Yihui Xie and Romain Lesur"
date: "`r Sys.Date()`"
output:
  pagedown::html_paged:
    toc: true
    # change to true for a self-contained document, but it'll be a litte slower for Pandoc to render
    self_contained: false
lot: true # insert a list of tables
lof: true # insert a list of figures
# uncomment this line to produce HTML and PDF in RStudio:
#knit: pagedown::chrome_print
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Introduction

This is an example of a multi-page HTML document with some options shown in YAML header. See https://pagedown.rbind.io for the full documentation. The rest of this document is random text.

# Random text

```{r, results='asis', echo = FALSE}
random_markdown = function(len = 100) {
  uri = knitr::image_uri(file.path(R.home('doc'), 'html', 'logo.jpg'))
  text = function(len) {
    trimws(paste(sample(c(letters, rep(' ', 10)), len, TRUE), collapse = ''))
  }
  id = function() paste(sample(letters, 8, TRUE), collapse = '')
  figure = function(i = id()) {
    sprintf('![(#fig:%s)The R logo.](%s){width=%d%%}', i, uri, sample(20:60, 1))
  }
  tab = paste(knitr::kable(head(mtcars[, 1:5])), collapse = '\n')
  table = function(i = id()) {
    c(sprintf('Table: (#tab:%s)A table example.', i), tab)
  }
  unlist(lapply(seq_len(len), function(i) {
    if (i %% 20 == 0) return(paste('#', text(sample(10:30, 1))))
    if (i %% 10 == 0) return(paste('##', text(sample(10:30, 1))))
    # insure some elements
    if (i == 3) return(text(50))
    if (i == 4) return(figure("md-fig"))
    if (i == 5) return(text(50))
    if (i == 6) return(table("md-tab"))
    # then random
    type = sample(1:3, 1, prob = c(.9, .03, .07))
    switch(type, text(sample(50:300, 1)), figure(), table())
  }))
}
cat(random_markdown(), sep = '\n\n')
```

# Knitr inserted Figures and tables

## Simple graphics

```{r simple-graphic, fig.cap = 'A very simple plot'}
plot(1)
```

## Simple tables

```{r simple-table}
knitr::kable(head(mtcars, 3), caption = "A Simple table")
```

---
title: "A letter of recommendation for Emi Tanaka"
author: "Yihui Xie"
output:
  pagedown::html_letter:
    self_contained: false
links-to-footnotes: true
paged-footnotes: true
# uncomment this line to produce HTML and PDF in RStudio:
#knit: pagedown::chrome_print
---

![The Posit logo](https://posit.co/wp-content/themes/Posit/dist/images/site-share-image.jpg){.logo}

::: from
Yihui Xie  
In the Middle of Nowhere  
Omaha, NE 68022  
Email: xie@yihui.name  
Homepage: https://yihui.name
:::


Hiring Manager  
School of Ninja, Hacker's University  
404 Not Found Road, Undefined 0x1234, NA

::: date
December 32, 2018
:::

Dear Hiring Manager,

It is my honor and pleasure to write a letter of recommendation^[Is this the first letter that contains GIFs ever in the history of human?] for Dr. Emi Tanaka, a great hacker in my eyes.

Emi said [it was heck of a praise](https://twitter.com/statsgen/status/1023118510745038848) when I said Earo was a very impressive hacker. Well, the fact is that I have been actually impressed many more times by Emi, which is probably because Earo is a relatively low-key hacker, and I have seen a lot of cool things Emi posted on Twitter this year. If I were to write a full letter about them, it would probably take me two whole days. Below is a one-hour attempt to show the many ways in which she can be so cool.

![The kunoichi theme for xaringan, made by Emi](https://github.com/emitanaka/ninja-theme/raw/master/docs/images/kunoichi-showcase.gif)

- She added the [ninja themes to **xaringan**](https://github.com/yihui/xaringan/pull/165).

- [The **anicon** package](https://twitter.com/statsgen/status/1021683550171496449) to insert animated icons or images or [text](https://twitter.com/statsgen/status/1010323499901075456) in R Markdown / **xaringan** / Shiny.

- [The **datalegreyar** package](https://twitter.com/statsgen/status/1018823773162618882) that uses a special typeface to represent data directly with text (which is an even smarter idea than sparklines). Magic.

- [She discovered the ultimate secret](https://twitter.com/statsgen/status/1012664098616590336) of the "Infinite Moon Reader" in **xaringan**: when an HTML page is automatically refreshed, the scroll position is preserved. This is extremely helpful when the document is long or has many pages, because otherwise you will have to scroll like crazy to the position where you last viewed. This feature is quite subtle and has been there for almost two years, and Emi was the first person (to my knowledge) who had discovered it and been excited about it.

- Obviously she is very good at hacking with CSS. [Look at her **xaringan** slides](https://twitter.com/statsgen/status/1009591310150299648) with the shiny (I mean sparkling) r-ladies theme, and the content is pretty good, too!

- [The joy over FiraCode](https://twitter.com/statsgen/status/993662737099079680) (as it was supported in RStudio).

- She wrote [one of the bestest **blogdown** tutorials](https://twitter.com/statsgen/status/993003327074926592) (the level of details is amazing).

- And [people truly appreciated](https://twitter.com/certifiedwaif/status/1029640487399915520) her **blogdown** seminar.

- Her website is one of the most beautiful websites I have seen. In particular, [everyone loved her showcase.](https://twitter.com/statsgen/status/990422351593406464) Did I mention that it was actually one of the four websites I highlighted [in the R Markdown book](https://bookdown.org/yihui/rmarkdown/basics-examples.html)?

- [The shiny custom loader.](https://twitter.com/statsgen/status/972727676459872256) Hahaha. [Super fun](https://twitter.com/statsgen/status/975321139550478336), and magic.

- [She can teach statistics courses.](https://twitter.com/statsgen/status/1001293550531694592) I barely remember what linear mixed models are.

- And [she just started answering](https://stackoverflow.com/a/51575804/559676) #rstats questions on Stack Overflow the other day! Thank you!!

Creative, enthusiastic, excellent taste of design, with great attention to details. That is my overall impression of this hacker.^[This letter was originally published in my blog at https://yihui.name/en/2018/07/emi-tanaka/.]

![The hacker's way of parking](https://slides.yihui.name/gif/impossible-parking.gif)

![Yihui's signature](https://user-images.githubusercontent.com/163582/47831497-c6656c00-dd5e-11e8-87b3-1f03fa97baa0.png)  
Yihui Xie  
Software Engineer  
RStudio, PBC

---
author:
  - name: Yihui Xie
    affiliation: RStudio
    address: |
      First line  
      Second line
    email: <name@company.com>
    url: <https://posit.co>
  - name: Romain Lesur
    affiliation: Affiliation
title: "A Capitalized Title: Something about a Package **foo**"
shorttitle: "**foo**:  A Capitalized Title"
abstract: |
  The abstract of the article.
# at least one keyword must be supplied
keywords: [keywords, not capitalized, "[Java]{.proglang}"]
links-to-footnotes: true
paged-footnotes: true
bibliography: index.bib
output: 
  pagedown::jss_paged:
    self_contained: false
# uncomment this line to produce HTML and PDF in RStudio:
#knit: pagedown::chrome_print
---

# Introduction

This template demonstrates some of the basic [markdown]{.proglang} you'll need to know to create a JSS article with **pagedown** [@R-pagedown].

## Code formatting

You should use **Pandoc**'s [markdown]{.proglang} but you also can use $\LaTeX$ commands:

* [Java]{.proglang} or $\proglang{Java}$ 
* **plyr**, [plyr]{.pkg} or $\pkg{plyr}$
* `print("abc")` or $\code{print("abc")}$

# [R]{.proglang} code

Can be inserted in regular [R Markdown]{.proglang} blocks.

```{r}
x <- 1:10
x
```

# Render the document

If you use the option `self_contained: false` (see line \#22 of this `Rmd` file), don't click on the `Knit` button in RStudio. Use instead the **xaringan** [@R-xaringan] RStudio add-in _Infinite Moon Reader_.

# References {-}

```{r, include=FALSE}
knitr::write_bib(c(.packages(), 'pagedown', 'xaringan'), 'index.bib')
```

---
title: "Test page breaks"
output:
  pagedown::html_paged:
    self_contained: false
    css: null
---

# First page

\newpage

# Second page

\pagebreak

# Third page {.page-break-after}

# Fourth page

# Fifth page {.page-break-before}

---
title: "A pagedown report with outlines"
author: "Yihui Xie and Romain Lesur"
date: "`r Sys.Date()`"
output:
  pagedown::html_paged:
    toc: true
    self_contained: false
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Title 1

Some content of title 1

# Title 2

## Title 2-1

Good

## Title 2-2

Wow

### Title 2-2-1

sub section

### Title 2-2-2

some words

# 中文 {#chinese1}

pagedown 不错

# 中文 带空格 {#chinese2}

很喜欢

## 中文次级标题 {#chinese2-1}

nice

---
title: "Untitled"
output:
  pagedown::html_paged:
    toc: true
    self_contained: false
---

# Chapter One

```{r}
summary(cars)
```

# Chapter Two

You can also embed plots, for example:

```{r, echo=FALSE}
plot(cars)
```

Note that the `echo = FALSE` parameter was added to the code chunk to prevent printing of the R code that generated the plot.

# Chapter Three

You can also print data frames

```{r}
mtcars
```

---
title: "Test reveal.js"
output: revealjs::revealjs_presentation
---

## R Markdown

This is an R Markdown presentation. Markdown is a simple formatting syntax for authoring HTML, PDF, and MS Word documents. For more details on using R Markdown see <http://rmarkdown.rstudio.com>.

When you click the **Knit** button a document will be generated that includes both content as well as the output of any embedded R code chunks within the document.

## Slide with Bullets

- Bullet 1
- Bullet 2
- Bullet 3

## Slide with R Code and Output

```{r}
summary(cars)
```

## Slide with Plot

```{r, echo=FALSE}
plot(cars)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paged.R
\name{html_paged}
\alias{html_paged}
\title{Create a paged HTML document suitable for printing}
\usage{
html_paged(
  ...,
  css = c("default-fonts", "default-page", "default"),
  theme = NULL,
  template = pkg_resource("html", "paged.html"),
  csl = NULL,
  front_cover = NULL,
  back_cover = NULL
)
}
\arguments{
\item{...}{Arguments passed to
\code{bookdown::\link[bookdown]{html_document2}}.}

\item{css}{A character vector of CSS and Sass file paths. If a path
does not contain the \file{.css}, \file{.sass}, or \file{.scss} extension,
it is assumed to be a built-in CSS file. For example, \code{default-fonts}
means the file\code{pagedown:::pkg_resource('css', 'default-fonts.css')}.
To see all built-in CSS files, run \code{pagedown:::list_css()}.}

\item{theme}{The Bootstrap theme. By default, Bootstrap is not used.}

\item{template}{The path to the Pandoc template to convert Markdown to HTML.}

\item{csl}{The path of the Citation Style Language (CSL) file used to format
citations and references (see the \href{https://pandoc.org/MANUAL.html#citations}{Pandoc documentation}).}

\item{front_cover, back_cover}{Paths or urls to image files to be used
as front or back covers. Theses images are available through CSS variables
(see Details).}
}
\value{
An R Markdown output format.
}
\description{
This is an output format based on \code{bookdown::html_document2} (which
means you can use those Markdown features added by \pkg{bookdown}). The HTML
output document is split into multiple pages via a JavaScript library
\pkg{paged.js}. These pages contain elements commonly seen in PDF documents,
such as page numbers and running headers.
}
\details{
When a path or an url is passed to the \code{front_cover} or \code{back_cover}
argument, several CSS variables are created. They are named \code{--front-cover}
and \code{--back-cover} and can be used as value for the CSS property \code{background-image}.
For example, \code{background-image: var(--front-cover);}. When a vector of
paths or urls is used as a value for \code{front_cover} or \code{back_cover},
the CSS variables are suffixed with an index: \code{--front-cover-1},
\code{--front-cover-2}, etc.
}
\references{
\url{https://pagedown.rbind.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resume.R
\name{html_resume}
\alias{html_resume}
\title{Create a resume in HTML}
\usage{
html_resume(
  ...,
  css = "resume",
  template = pkg_resource("html", "resume.html"),
  number_sections = FALSE,
  fig_caption = FALSE
)
}
\arguments{
\item{..., css, template, number_sections, fig_caption}{See
\code{\link{html_paged}()}.}
}
\value{
An R Markdown output format.
}
\description{
This output format is based on Min-Zhong Lu's HTML/CSS in the Github repo
\url{https://github.com/mnjul/html-resume}. See
\url{https://pagedown.rbind.io/html-resume/} for an example.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paged.R
\name{book_crc}
\alias{book_crc}
\title{Create a book for Chapman & Hall/CRC}
\usage{
book_crc(..., css = c("crc-page", "default-page", "default", "crc"))
}
\arguments{
\item{..., css}{Arguments passed to \code{\link{html_paged}()}.}
}
\value{
An R Markdown output format.
}
\description{
This output format is similar to \code{\link{html_paged}}. The only
difference is in the default stylesheets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paged.R
\name{thesis_paged}
\alias{thesis_paged}
\title{Create a paged HTML thesis document suitable for printing}
\usage{
thesis_paged(
  ...,
  css = c("thesis"),
  template = pkg_resource("html", "thesis.html")
)
}
\arguments{
\item{..., css, template}{Arguments passed to \code{\link{html_paged}()}.}
}
\value{
An R Markdown output format.
}
\description{
This output format is similar to \code{\link{html_paged}}. The only
difference is in the default stylesheets and Pandoc template. See
\url{https://pagedown.rbind.io/thesis-paged/} for an example.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/card.R
\name{business_card}
\alias{business_card}
\title{Create business cards}
\usage{
business_card(template = pkg_resource("html", "card.html"))
}
\arguments{
\item{template}{The path to the Pandoc template to convert Markdown to HTML.}
}
\value{
An R Markdown output format.
}
\description{
This output format is based on an example in the Github repo
\url{https://github.com/RelaxedJS/ReLaXed-examples}. See
\url{https://pagedown.rbind.io/business-card/} for an example.
}
\examples{
pagedown::business_card()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paged.R
\name{html_letter}
\alias{html_letter}
\title{Create a letter in HTML}
\usage{
html_letter(..., css = c("default", "letter"), fig_caption = FALSE)
}
\arguments{
\item{..., css, fig_caption}{Arguments passed to \code{\link{html_paged}()}.}
}
\value{
An R Markdown output format.
}
\description{
This output format is similar to \code{html_paged}. The only differences are
in the default stylesheets and the default value of the \code{fig_caption}
parameter which is set to \code{FALSE}. See
\url{https://pagedown.rbind.io/html-letter/} for an example.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/poster.R
\name{poster_relaxed}
\alias{poster_relaxed}
\alias{poster_jacobs}
\title{Create posters in HTML}
\usage{
poster_relaxed(
  ...,
  css = "poster-relaxed",
  template = pkg_resource("html", "poster-relaxed.html"),
  number_sections = FALSE
)

poster_jacobs(
  ...,
  css = "poster-jacobs",
  template = pkg_resource("html", "poster-jacobs.html")
)
}
\arguments{
\item{..., css, template, number_sections}{See \code{\link{html_paged}()}.}
}
\value{
An R Markdown output format.
}
\description{
The output format \code{poster_relaxed()} is based on an example in the
Github repo \url{https://github.com/RelaxedJS/ReLaXed-examples}. See
\url{https://pagedown.rbind.io/poster-relaxed/} for an example.

The output format \code{poster_jacobs()} mimics the style of the
  \dQuote{Jacobs Landscape Poster LaTeX Template Version 1.0} at
  \url{https://www.overleaf.com/gallery/tagged/poster}. See
  \url{https://pagedown.rbind.io/poster-jacobs/} for an example.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{find_chrome}
\alias{find_chrome}
\title{Find Google Chrome, Microsoft Edge or Chromium in the system}
\usage{
find_chrome()
}
\value{
A character string.
}
\description{
On Windows, this function tries to find Chrome or Edge from the registry. On
macOS, it returns a hard-coded path of Chrome under \file{/Applications}. On
Linux, it searches for \command{chromium-browser} and \command{google-chrome}
from the system's \var{PATH} variable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pagedown-package.R
\docType{package}
\name{pagedown-package}
\alias{pagedown}
\alias{pagedown-package}
\title{pagedown: Paginate the HTML Output of R Markdown with CSS for Print}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Use the paged media properties in CSS and the JavaScript library 'paged.js' to split the content of an HTML document into discrete pages. Each page can have its page size, page numbers, margin boxes, and running headers, etc. Applications of this package include books, letters, reports, papers, business cards, resumes, and posters.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rstudio/pagedown}
  \item Report bugs at \url{https://github.com/rstudio/pagedown/issues}
}

}
\author{
\strong{Maintainer}: Yihui Xie \email{xie@yihui.name} (\href{https://orcid.org/0000-0003-0645-5666}{ORCID})

Authors:
\itemize{
  \item Romain Lesur (\href{https://orcid.org/0000-0002-0721-5595}{ORCID}) [copyright holder]
  \item Brent Thorne (\href{https://orcid.org/0000-0002-1099-3857}{ORCID})
  \item Xianying Tan (\href{https://orcid.org/0000-0002-6072-3521}{ORCID})
}

Other contributors:
\itemize{
  \item Christophe Dervieux (\href{https://orcid.org/0000-0003-4474-2498}{ORCID}) [contributor]
  \item Atsushi Yasumoto (\href{https://orcid.org/0000-0002-8335-495X}{ORCID}) [contributor]
  \item Posit Software, PBC [copyright holder, funder]
  \item Adam Hyde (paged.js in resources/js/) [contributor]
  \item Min-Zhong Lu (resume.css in resources/css/) [contributor]
  \item Zulko (poster-relaxed.css in resources/css/) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{chrome_print}
\alias{chrome_print}
\title{Print a web page to PDF or capture a screenshot using the headless Chrome}
\usage{
chrome_print(
  input,
  output = xfun::with_ext(input, format),
  wait = 2,
  browser = "google-chrome",
  format = c("pdf", "png", "jpeg"),
  options = list(),
  selector = "body",
  box_model = c("border", "content", "margin", "padding"),
  scale = 1,
  work_dir = tempfile(),
  timeout = 30,
  extra_args = c("--disable-gpu"),
  verbose = 0,
  async = FALSE,
  outline = gs_available(),
  encoding
)
}
\arguments{
\item{input}{A URL or local file path to an HTML page, or a path to a local
file that can be rendered to HTML via \code{rmarkdown::\link[rmarkdown]{render}()}
(e.g., an R Markdown document or an R script). If the \code{input} is to be
rendered via \code{rmarkdown::render()} and you need to pass any arguments
to it, you can pass the whole \code{render()} call to
\code{chrome_print()}, e.g., if you need to use the \code{params} argument:
\code{pagedown::chrome_print(rmarkdown::render('input.Rmd', params =
list(foo = 1:10)))}. This is because \code{render()} returns the HTML file,
which can be passed to \code{chrome_print()}.}

\item{output}{The output filename. For a local web page \file{foo/bar.html},
the default PDF output is \file{foo/bar.pdf}; for a remote URL
\file{https://www.example.org/foo/bar.html}, the default output will be
\file{bar.pdf} under the current working directory. The same rules apply
for screenshots.}

\item{wait}{The number of seconds to wait for the page to load before
printing (in certain cases, the page may not be immediately ready for
printing, especially there are JavaScript applications on the page, so you
may need to wait for a longer time).}

\item{browser}{Path to Google Chrome, Microsoft Edge or Chromium. This
function will try to find it automatically via \code{\link{find_chrome}()}
if the path is not explicitly provided and the environment variable
\code{PAGEDOWN_CHROME} is not set.}

\item{format}{The output format.}

\item{options}{A list of page options. See
\code{https://chromedevtools.github.io/devtools-protocol/tot/Page#method-printToPDF}
 for the full list of options for PDF output, and
\code{https://chromedevtools.github.io/devtools-protocol/tot/Page#method-captureScreenshot}
 for options for screenshots. Note that for PDF output, we have changed the
defaults of \code{printBackground} (\code{TRUE}),
\code{preferCSSPageSize} (\code{TRUE}) and when available
\code{transferMode} (\code{ReturnAsStream}) in this function.}

\item{selector}{A CSS selector used when capturing a screenshot.}

\item{box_model}{The CSS box model used when capturing a screenshot.}

\item{scale}{The scale factor used for screenshot.}

\item{work_dir}{Name of headless Chrome working directory. If the default
temporary directory doesn't work, you may try to use a subdirectory of your
home directory.}

\item{timeout}{The number of seconds before canceling the document
generation. Use a larger value if the document takes longer to build.}

\item{extra_args}{Extra command-line arguments to be passed to Chrome.}

\item{verbose}{Level of verbosity: \code{0} means no messages; \code{1} means
to print out some auxiliary messages (e.g., parameters for capturing
screenshots); \code{2} (or \code{TRUE}) means all messages, including those
from the Chrome processes and WebSocket connections.}

\item{async}{Execute \code{chrome_print()} asynchronously? If \code{TRUE},
\code{chrome_print()} returns a \code{\link[promises]{promise}} value (the
\pkg{promises} package has to be installed in this case).}

\item{outline}{If not \code{FALSE}, \code{chrome_print()} will add the
bookmarks to the generated \code{pdf} file, based on the table of contents
informations. This feature is only available for output formats based on
\code{\link{html_paged}}. It is enabled by default, as long as the
Ghostscript executable can be detected by \code{\link[tools]{find_gs_cmd}}.}

\item{encoding}{Not used. This argument is required by RStudio IDE.}
}
\value{
Path of the output file (invisibly). If \code{async} is \code{TRUE},
  this is a \code{\link[promises]{promise}} value.
}
\description{
Print an HTML page to PDF or capture a PNG/JPEG screenshot through the Chrome
DevTools Protocol. Google Chrome or Microsoft Edge (or Chromium on Linux)
must be installed prior to using this function.
}
\references{
\url{https://developer.chrome.com/blog/headless-chrome/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paged.R
\name{jss_paged}
\alias{jss_paged}
\title{Create an article for the Journal of Statistical Software}
\usage{
jss_paged(
  ...,
  css = c("jss-fonts", "jss-page", "jss"),
  template = pkg_resource("html", "jss_paged.html"),
  csl = pkg_resource("csl", "journal-of-statistical-software.csl"),
  highlight = NULL,
  pandoc_args = NULL
)
}
\arguments{
\item{..., css, template, csl, highlight, pandoc_args}{Arguments passed to \code{\link{html_paged}()}.}
}
\value{
An R Markdown output format.
}
\description{
This output format is similar to \code{\link{html_paged}}.
}
